/**
 * Opaque.java
 *
 * (C) Copyright IBM Corp. 2005, 2006
 *
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE COMMON PUBLIC LICENSE 
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE 
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Common Public License from
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * @author: Roberto Pineiro, IBM, roberto.pineiro@us.ibm.com  
 * @author: Chung-hao Tan, IBM ,chungtan@us.ibm.com
 * 
 * Change History
 * Flag       Date        Prog         Description
 *------------------------------------------------------------------------------- 
 * 1516246    2006-07-22  lupusalex    Integrate SLP client code
 */

package org.sblim.slp;

import java.util.Arrays;

/**
 * The Opaque class is the Java representation of the OPAQUE attribute type as
 * defined by RFC 3224. It encapsulates a byte array in an object.
 */
public class Opaque {

	private byte iDataBytes[];

	/**
	 * Ctor.
	 * 
	 * @param pBytes
	 *            The byte array to encapsulate
	 */
	public Opaque(byte[] pBytes) {
		if (pBytes == null) throw new IllegalArgumentException();
		iDataBytes = pBytes;
	}

	/**
	 * Gets the data bytes
	 * 
	 * @return Returns the dataBytes.
	 */
	public byte[] getDataBytes() {
		return iDataBytes;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object o) {
		if (o == this) return true;
		if (!(o instanceof Opaque)) return false;

		Opaque that = (Opaque) o;

		return Arrays.equals(this.iDataBytes, that.iDataBytes);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	public String toString() {

		String s = null;
		StringBuffer stringbuffer = new StringBuffer();
		stringbuffer.append("\\ff");

		for (int i = 0; i < iDataBytes.length; i++) {
			byte b = iDataBytes[i];
			s = Integer.toHexString(b & 0xff);
			stringbuffer.append('\\');
			if (b < 16) stringbuffer.append('0');
			stringbuffer.append(s);
		}

		return stringbuffer.toString();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		return iDataBytes.hashCode();
	}

}
