/**
 * ServiceLocationEnumeration.java
 *
 * (C) Copyright IBM Corp. 2005, 2006
 *
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE COMMON PUBLIC LICENSE 
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE 
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Common Public License from
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * @author: Roberto Pineiro, IBM, roberto.pineiro@us.ibm.com  
 * @author: Chung-hao Tan, IBM ,chungtan@us.ibm.com
 * 
 * Change History
 * Flag       Date        Prog         Description
 *------------------------------------------------------------------------------- 
 * 1516246    2006-07-22  lupusalex    Integrate SLP client code
 */

package org.sblim.slp;

import java.util.Enumeration;
import java.util.NoSuchElementException;

/**
 * The ServiceLocationEnumeration class is the return type for all Locator SLP
 * operations. The Java API library may implement this class to block until
 * results are available from the SLP operation, so that the client can achieve
 * asynchronous operation by retrieving results from the enumeration in a
 * separate thread. Clients use the superclass nextElement() method if they are
 * unconcerned with SLP exceptions (this method will never ever throw one).
 */
public interface ServiceLocationEnumeration extends Enumeration {

	/**
	 * Return the next value or block until it becomes available.
	 * 
	 * @return The next value
	 * @throws ServiceLocationException
	 *             Thrown if the SLP operation encounters an error.
	 * @throws NoSuchElementException
	 *             If there are no more elements to return.
	 */
	public abstract Object next() throws ServiceLocationException, NoSuchElementException;

}
