/**
 * AdvertiserImpl.java
 *
 * (C) Copyright IBM Corp. 2006
 *
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE COMMON PUBLIC LICENSE 
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE 
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Common Public License from
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * @author: Roberto Pineiro, IBM, roberto.pineiro@us.ibm.com  
 * @author: Chung-hao Tan, IBM ,chungtan@us.ibm.com
 * 
 * Change History
 * Flag       Date        Prog         Description
 *------------------------------------------------------------------------------- 
 * 1516246    2006-07-22  lupusalex    Integrate SLP client code
 * 1535793    2006-09-14  lupusalex    Fix&Integrate CIM&SLP configuration classes
 */

package org.sblim.slp.internal;

import java.util.Locale;
import java.util.Vector;

import org.sblim.slp.Advertiser;
import org.sblim.slp.ServiceLocationException;
import org.sblim.slp.ServiceURL;

/**
 * The class AdvertiserImpl implements the Advertister interface as defined by
 * RFC 3224
 */
public class AdvertiserImpl implements Advertiser {

	private Locale iLocale;

	public AdvertiserImpl(Locale pLocale) {
		iLocale = pLocale;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.slp.Advertiser#getLocale()
	 */
	public Locale getLocale() {
		return iLocale;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.slp.Advertiser#register(com.ibm.slp.ServiceURL,
	 *      java.util.Vector)
	 */
	public void register(ServiceURL pServiceUrl, Vector pAttibutes) throws ServiceLocationException {

		if (pServiceUrl == null) throw new IllegalArgumentException();
		if (pAttibutes == null) throw new IllegalArgumentException();

		Vector scopes = SLPConfig.getGlobalSLPConfig().getConfiguredScopes();
		SLPServiceRegistration csrvreg = new SLPServiceRegistration(true, iLocale, pServiceUrl,
				scopes, pAttibutes);

		ServiceLocationMessage srvlocmsg = MessageDispatcherStreamed.dispatchStreamMsg(SLPConfig
				.getLoopback(), csrvreg, true);

		if (srvlocmsg == null || (srvlocmsg.getHeader().getFunctionCode() != 5)) throw new ServiceLocationException(
				ServiceLocationException.NETWORK_ERROR);

		short errorCode = srvlocmsg.getErrorCode();
		if (errorCode != 0) throw new ServiceLocationException(errorCode);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.slp.Advertiser#deregister(com.ibm.slp.ServiceURL)
	 */
	public void deregister(ServiceURL pServiceUrl) throws ServiceLocationException {
		if (pServiceUrl == null) throw new IllegalArgumentException("URL");

		Vector scopes = SLPConfig.getGlobalSLPConfig().getConfiguredScopes();

		SLPServiceDeregistration csrvdereg = new SLPServiceDeregistration(iLocale, pServiceUrl,
				scopes, null);
		ServiceLocationMessage srvlocmsg = MessageDispatcherStreamed.dispatchStreamMsg(SLPConfig
				.getLoopback(), csrvdereg, true);

		short errorCode = srvlocmsg.getErrorCode();
		if (errorCode != 0) throw new ServiceLocationException(errorCode);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.slp.Advertiser#addAttributes(com.ibm.slp.ServiceURL,
	 *      java.util.Vector)
	 */
	public void addAttributes(ServiceURL pUrl, Vector pAttributes) throws ServiceLocationException {
		throw new RuntimeException("not implemented");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.slp.Advertiser#deleteAttributes(com.ibm.slp.ServiceURL,
	 *      java.util.Vector)
	 */
	public void deleteAttributes(ServiceURL pUrl, Vector pAttributeIds)
			throws ServiceLocationException {
		throw new RuntimeException("not implemented");
	}
}
