/**
 * DebugInputStream.java
 *
 * (C) Copyright IBM Corp. 2006
 *
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE COMMON PUBLIC LICENSE 
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE 
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Common Public License from
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * @author: Roberto Pineiro, IBM, roberto.pineiro@us.ibm.com  
 * @author: Chung-hao Tan, IBM ,chungtan@us.ibm.com
 * 
 * Change History
 * Flag       Date        Prog         Description
 *------------------------------------------------------------------------------- 
 * 1516246    2006-07-22  lupusalex    Integrate SLP client code
 */

package org.sblim.slp.internal;

import java.io.FilterInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

public class DebugInputStream extends FilterInputStream {

	private byte[] iBuffer;

	private boolean iBuffered;

	private int iCurrent = 0;

	private int iMaxLen = 0;

	private OutputStream iOut;

	private static final char[] HEX = "01234567890ABCEDF".toCharArray();

	public DebugInputStream(InputStream pIn, OutputStream pOut) {
		super(pIn);
		iBuffer = new byte[512];
		iBuffered = false;
		iOut = pOut;
	}

	private void buffer() throws IOException {

		iBuffered = true;
		int total;
		while ((total = in.read(iBuffer, iMaxLen, iBuffer.length - iMaxLen)) > -1) {
			iMaxLen += total;
			if (iMaxLen == iBuffer.length) {
				byte b[] = new byte[iBuffer.length << 1];
				System.arraycopy(iBuffer, 0, b, 0, iBuffer.length);
				iBuffer = b;
			}
		}
		if (iOut != null) {
			iOut.write("-----------------------------\n".getBytes());
			iOut.write("Dumping raw data from stream:\n".getBytes());
			iOut.write(iBuffer, 0, iMaxLen);
			iOut.write("\nHex values=\n".getBytes());
			iOut.write(getHexArray(iBuffer).getBytes());
			iOut.write("\n".getBytes());
			iOut.write("-----------------------------\n".getBytes());
		}
	}

	private String getHexArray(byte[] pBuffer) {
		if (pBuffer == null) return "null";
		StringBuffer sb = new StringBuffer();
		for (int i = 0; i < pBuffer.length; i++) {
			byte b = pBuffer[i];
			sb.append(HEX[(b >> 4) & 7]);
			sb.append(HEX[(b) & 7]);
			sb.append(' ');
		}
		return sb.toString();
	}

	public synchronized int read() throws IOException {
		if (!iBuffered) buffer();

		if (iCurrent >= iMaxLen) return -1;
		return iBuffer[iCurrent++] & 0xFF;
	}

	public synchronized int read(byte[] pBuffer, int pOffset, int pLength) throws IOException {
		if (pBuffer == null) {
			throw new NullPointerException();
		} else if ((pOffset < 0) || (pOffset > pBuffer.length) || (pLength < 0)
				|| ((pOffset + pLength) > pBuffer.length) || ((pOffset + pLength) < 0)) {
			throw new IndexOutOfBoundsException();
		} else if (pLength == 0) { return 0; }

		int c = read();
		if (c == -1) { return -1; }
		pBuffer[pOffset] = (byte) c;

		int i = 1;
		for (; i < pLength; i++) {
			c = read();
			if (c == -1) {
				break;
			}
			if (pBuffer != null) {
				pBuffer[pOffset + i] = (byte) c;
			}
		}
		return i;
	}
}
