/**
 * LocatorImplStreamed.java
 *
 * (C) Copyright IBM Corp. 2005, 2006
 *
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE COMMON PUBLIC LICENSE 
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE 
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Common Public License from
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * @author: Roberto Pineiro, IBM, roberto.pineiro@us.ibm.com  
 * @author: Chung-hao Tan, IBM ,chungtan@us.ibm.com
 * 
 * Change History
 * Flag       Date        Prog         Description
 *------------------------------------------------------------------------------- 
 * 16966      2005-04-29  thschaef     Add findServices() signature with DAs as parameter
 * 1516246    2006-07-22  lupusalex    Integrate SLP client code
 * 1535793    2006-09-14  lupusalex    Fix&Integrate CIM&SLP configuration classes
 */

package org.sblim.slp.internal;

import java.util.Enumeration;
import java.util.Locale;
import java.util.Vector;

import org.sblim.slp.Locator;
import org.sblim.slp.ServiceLocationEnumeration;
import org.sblim.slp.ServiceLocationException;
import org.sblim.slp.ServiceType;
import org.sblim.slp.ServiceURL;

/**
 * The class LocatorImplStreamed implements the Locator interface as defined by
 * RFC 3224
 */
public class LocatorImplStreamed implements Locator {

	// private SLPConfig iConfig = SLPConfig.getSLPConfig();

	private Locale iLocale;

	public LocatorImplStreamed(Locale pLocale) {
		iLocale = pLocale;
	}

	private Enumeration createMessages(int pFunctionCode, String pQueryString,
			String pNamingAuthority, ServiceURL pServiceUrl, ServiceType pServiceType,
			Vector pScopes, Vector pAttributeIds, Vector pDiscoveryAgents)
			throws ServiceLocationException {

		// Validate scopes
		// validateScopes(vector, locale);

		ServiceLocationMessage request = null;

		// TODO get the list of DA from the config, and the Active DA discovery
		// engine
		Vector daAddresses;
		if (pDiscoveryAgents != null) daAddresses = pDiscoveryAgents;
		else daAddresses = SLPConfig.getGlobalSLPConfig().getPreconfiguredDAs();
		Vector saScopes = new Vector();
		// TODO filter DA using scopes
		saScopes.addAll(pScopes);

		if ((pFunctionCode == ServiceLocationHeader.SRVRQST || pFunctionCode == ServiceLocationHeader.ATTRRQST)

				&& pServiceType != null
				&& (pServiceType.equals(SLPDefaults.DA_SERVICE_TYPE) || pServiceType
						.equals(SLPDefaults.SA_SERVICE_TYPE))) {

			String query = "";
			if (pFunctionCode == ServiceLocationHeader.SRVRQST && pQueryString != null) query = pQueryString;

			request = new SLPServiceRequestMessage(iLocale, pServiceType, pScopes, query);
		} else {
			switch (pFunctionCode) {
				case ServiceLocationHeader.SRVRPLY:
				case ServiceLocationHeader.SRVREG:
				case ServiceLocationHeader.SRVDEREG:
				case ServiceLocationHeader.SRVACK:
				case ServiceLocationHeader.ATTRRPLY:
				case ServiceLocationHeader.DAADVERT:
				default:
					break;
				case ServiceLocationHeader.SRVTYPERQST:
					request = new SLPServiceTypeRequestMessage(iLocale, pNamingAuthority, pScopes);
					break;

				case ServiceLocationHeader.SRVRQST:
					request = new SLPServiceRequestMessage(iLocale, pServiceType, pScopes,
							pQueryString);
					break;
				case ServiceLocationHeader.ATTRRQST:
					if (pServiceUrl != null) request = new SLPAttributeRequestMessage(iLocale,
							pServiceUrl, pScopes, pAttributeIds);
					else request = new SLPAttributeRequestMessage(iLocale, pServiceType, pScopes,
							pAttributeIds);
					break;
			}
		}
		return ServiceLocationEnumeratorStreamed.dispatchUAMsg(daAddresses, request, SLPConfig
				.getMulticastAddress());
	}

	public synchronized ServiceLocationEnumeration findServices(ServiceType pServiceType,
			Vector pScopes, String pQuery) throws ServiceLocationException {

		if (pServiceType == null) throw new IllegalArgumentException("null type");
		if (pScopes == null) throw new IllegalArgumentException("null scopes");
		if (pQuery == null) throw new IllegalArgumentException("null query");

		return new ServiceLocationServiceEnumeration(createMessages(1, pQuery, null, null,
				pServiceType, pScopes, null, null));
	}

	public synchronized ServiceLocationEnumeration findServices(ServiceType pServiceType,
			Vector pScopes, String pQuery, Vector pDiscoveryAgents) throws ServiceLocationException {

		if (pServiceType == null) throw new IllegalArgumentException("null type");
		if (pScopes == null) throw new IllegalArgumentException("null scopes");
		if (pQuery == null) throw new IllegalArgumentException("null query");

		return new ServiceLocationServiceEnumeration(createMessages(1, pQuery, null, null,
				pServiceType, pScopes, null, pDiscoveryAgents));
	}

	public Locale getLocale() {
		return iLocale;
	}

	public ServiceLocationEnumeration findServiceTypes(String pNamingAuthority, Vector pScopes)
			throws ServiceLocationException {
		if (pNamingAuthority == null) throw new IllegalArgumentException("Naming Authority");
		if (pScopes == null) throw new IllegalArgumentException("scopes");

		return new ServiceLocationServiceTypeEnumeration(createMessages(9, null, pNamingAuthority,
				null, null, pScopes, null, null));
	}

	public ServiceLocationEnumeration findServiceTypes(String pNamingAuthority, Vector pScopes,
			Vector pDirectoryAgent) throws ServiceLocationException {
		if (pNamingAuthority == null) throw new IllegalArgumentException("Naming Authority");
		if (pScopes == null) throw new IllegalArgumentException("scopes");

		return new ServiceLocationServiceTypeEnumeration(createMessages(9, null, pNamingAuthority,
				null, null, pScopes, null, pDirectoryAgent));
	}

	public ServiceLocationEnumeration findAttributes(ServiceURL pServiceUrl, Vector pScopes,
			Vector pAttributeIds) throws ServiceLocationException {
		if (pServiceUrl == null) throw new IllegalArgumentException("serviceurl");
		if (pScopes == null) throw new IllegalArgumentException("scopes");
		if (pAttributeIds == null) throw new IllegalArgumentException("attributeIds");

		return new ServiceLocationAttributeEnumeration2(pServiceUrl, createMessages(6, null, null,
				pServiceUrl, null, pScopes, pAttributeIds, null));
	}

	public ServiceLocationEnumeration findAttributes(ServiceURL pServiceUrl, Vector pScopes,
			Vector pAttributeIds, Vector pDirectoryAgents) throws ServiceLocationException {
		if (pServiceUrl == null) throw new IllegalArgumentException("serviceurl");
		if (pScopes == null) throw new IllegalArgumentException("scopes");
		if (pAttributeIds == null) throw new IllegalArgumentException("attributeIds");

		return new ServiceLocationAttributeEnumeration2(pServiceUrl, createMessages(6, null, null,
				pServiceUrl, null, pScopes, pAttributeIds, pDirectoryAgents));
	}

	public ServiceLocationEnumeration findAttributes(ServiceType pServiceType, Vector pScopes,
			Vector pAttributeIds) throws ServiceLocationException {
		if (pServiceType == null) throw new IllegalArgumentException("servicetype");
		if (pScopes == null) throw new IllegalArgumentException("scopes");
		if (pAttributeIds == null) throw new IllegalArgumentException("attributeIds");

		return new ServiceLocationAttributeEnumeration(createMessages(6, (String) null,
				(String) null, (ServiceURL) null, pServiceType, pScopes, pAttributeIds,
				(Vector) null));
	}

	public ServiceLocationEnumeration findAttributes(ServiceType pServiceType, Vector pScopes,
			Vector pAttributeIds, Vector pDirectoryAgents) throws ServiceLocationException {
		if (pServiceType == null) throw new IllegalArgumentException("servicetype");
		if (pScopes == null) throw new IllegalArgumentException("scopes");
		if (pAttributeIds == null) throw new IllegalArgumentException("attributeIds");

		return new ServiceLocationAttributeEnumeration(createMessages(6, null, null, null,
				pServiceType, pScopes, pAttributeIds, pDirectoryAgents));
	}

}

class ServiceLocationServiceTypeEnumeration implements ServiceLocationEnumeration {

	private Enumeration iEnumeration;

	private int iTotalResults = 0;

	private int iMaxResults = SLPConfig.getGlobalSLPConfig().getMaximumResults();

	private Vector iList = new Vector();

	private Vector iResult = new Vector();

	public ServiceLocationServiceTypeEnumeration(Enumeration pEnumeration) {
		iEnumeration = pEnumeration;
	}

	public Object next() throws ServiceLocationException {
		getNext();
		if (iList.size() == 0) { return null; }
		return iList.remove(0);
	}

	public boolean hasMoreElements() {
		getNext();
		return iList.size() > 0 && iTotalResults < iMaxResults;
	}

	private void getNext() {
		if (iList.size() == 0) {
			while (iList.size() == 0 && iEnumeration.hasMoreElements()) {
				int maxResults = SLPConfig.getGlobalSLPConfig().getMaximumResults();

				SLPServiceTypeReplyMessage srvtype = (SLPServiceTypeReplyMessage) iEnumeration
						.nextElement();

				short e = srvtype.getErrorCode();
				if (e != 0 && !iEnumeration.hasMoreElements() && iTotalResults == 0) { throw new ServiceLocationException(
						e); }
				Vector urlList = srvtype.getServiceTypes();
				if (urlList != null) {
					int size = urlList.size();
					for (int m = 0; m < size; m++) {
						Object o = urlList.elementAt(m);

						if (!iResult.contains(o) && iResult.size() < maxResults) {
							iResult.addElement(o);
							iList.addElement(o);
						}
					}
				}
			}
		}
	}

	public Object nextElement() {
		getNext();
		if (iList.size() > 0) {
			iTotalResults++;
			return iList.remove(0);
		}
		return null;
	}
}

class ServiceLocationServiceEnumeration implements ServiceLocationEnumeration {

	private Enumeration iEnumeration;

	private int iTotalResults = 0;

	private int iMaxResults = SLPConfig.getGlobalSLPConfig().getMaximumResults();

	private Vector iList = new Vector();

	private Vector iUrlList = null;

	private Vector iResult = new Vector();

	public ServiceLocationServiceEnumeration(Enumeration pEnumeration) {
		iEnumeration = pEnumeration;
	}

	public Object next() throws ServiceLocationException {
		getNext();
		if (iList.size() == 0) { return null; }
		return iList.remove(0);
	}

	public boolean hasMoreElements() {
		getNext();
		return iList.size() > 0 && iTotalResults < iMaxResults;
	}

	private void getNext() {
		if (iList.size() == 0) {
			while (iList.size() == 0 && iEnumeration.hasMoreElements()) {
				ServiceLocationMessage srvlocmsg = (ServiceLocationMessage) iEnumeration
						.nextElement();
				short n;
				if ((n = srvlocmsg.getErrorCode()) != 0 && !iEnumeration.hasMoreElements()
						&& iTotalResults == 0) throw new ServiceLocationException(n);

				iUrlList = null;

				if (srvlocmsg instanceof SLPServiceReplyMessage) {
					iUrlList = ((SLPServiceReplyMessage) srvlocmsg).getServiceURLs();
				} else if (srvlocmsg instanceof SLPSAAdvert) {
					iUrlList = new Vector();
					iUrlList.addElement(((SLPSAAdvert) srvlocmsg).getUrl());
				} else if (srvlocmsg instanceof SLPDAAdvert) {
					iUrlList = new Vector();
					iUrlList.addElement(((SLPDAAdvert) srvlocmsg).getUrl());
				}

				if (iUrlList != null) {
					int size = iUrlList.size();
					for (int m = 0; m < size; m++) {
						Object o = iUrlList.elementAt(m);

						if (!iResult.contains(o) && iResult.size() < iMaxResults) {
							iResult.addElement(o);
							iList.addElement(o);
						}
					}
				}
			}
		}
	}

	public Object nextElement() {
		getNext();
		if (iList.size() > 0) {
			iTotalResults++;
			return iList.remove(0);
		}
		return null;
	}
}

class ServiceLocationAttributeEnumeration implements ServiceLocationEnumeration {

	private Enumeration iEnumeration;

	private int iTotalResults = 0;

	private int iMaxResults = SLPConfig.getGlobalSLPConfig().getMaximumResults();

	private Vector iList = new Vector();

	public ServiceLocationAttributeEnumeration(Enumeration pEnumeration) {
		iEnumeration = pEnumeration;
	}

	public Object next() throws ServiceLocationException {
		getNext();
		if (iList.size() == 0) { return null; }
		return iList.remove(0);
	}

	public boolean hasMoreElements() {
		getNext();
		return iTotalResults < iMaxResults && iList.size() > 0;
	}

	private void getNext() {
		if (iList.size() == 0) {
			while (iList.size() == 0 && iEnumeration.hasMoreElements()) {
				ServiceLocationMessage srvlocmsg = (ServiceLocationMessage) iEnumeration
						.nextElement();

				short e = srvlocmsg.getErrorCode();
				if (e != 0 && !iEnumeration.hasMoreElements() && iTotalResults == 0) { throw new ServiceLocationException(
						e); }

				Vector attrList = null;

				if (srvlocmsg instanceof SLPAttributeReplyMessage) attrList = ((SLPAttributeReplyMessage) srvlocmsg)
						.getAttributeList();
				else if (srvlocmsg instanceof SLPSAAdvert) attrList = ((SLPSAAdvert) srvlocmsg)
						.getAttributes();
				else if (srvlocmsg instanceof SLPDAAdvert) attrList = ((SLPDAAdvert) srvlocmsg)
						.getAttributes();

				iList.addAll(attrList);
			}
		}
	}

	public Object nextElement() {
		getNext();
		if (iList.size() == 0) { return null; }
		iTotalResults++;
		return iList.remove(0);
	}

};

class ServiceLocationAttributeEnumeration2 implements ServiceLocationEnumeration {

	private Enumeration iEnumeration;

	private int iTotalResults = 0;

	private int iMaxResults = SLPConfig.getGlobalSLPConfig().getMaximumResults();

	private Vector iList = new Vector();

	private ServiceURL iServiceUrl;

	public ServiceLocationAttributeEnumeration2(ServiceURL pServiceurl, Enumeration pEnumeration) {
		iEnumeration = pEnumeration;
		iServiceUrl = pServiceurl;
	}

	public Object next() throws ServiceLocationException {
		getNext();
		if (iList.size() == 0) { return null; }
		return iList.remove(0);
	}

	public boolean hasMoreElements() {
		getNext();
		return iTotalResults < iMaxResults && iList.size() > 0;
	}

	private void getNext() {
		if (iList.size() == 0) {
			while (iList.size() == 0 && iEnumeration.hasMoreElements()) {
				ServiceLocationMessage srvlocmsg = (ServiceLocationMessage) iEnumeration
						.nextElement();

				short e = srvlocmsg.getErrorCode();
				if (e != 0 && !iEnumeration.hasMoreElements() && iTotalResults == 0) { throw new ServiceLocationException(
						e); }

				// short e = srvloc.getErrorCode();
				// if (e != 0 && responseMsg.size() == 1)
				// throw new ServiceLocationException(e);
				//
				//

				Vector attrList = null;

				if (srvlocmsg instanceof SLPAttributeReplyMessage) attrList = ((SLPAttributeReplyMessage) srvlocmsg)
						.getAttributeList();
				else if (srvlocmsg instanceof SLPSAAdvert) {
					SLPSAAdvert csaadvert = (SLPSAAdvert) srvlocmsg;
					if (!iServiceUrl.equals(csaadvert.getUrl())) continue;
					attrList = csaadvert.getAttributes();
				} else if (srvlocmsg instanceof SLPDAAdvert) {
					SLPDAAdvert cdaadvert = (SLPDAAdvert) srvlocmsg;
					if (!iServiceUrl.equals(cdaadvert.getUrl())) continue;
					attrList = cdaadvert.getAttributes();
				}

				iList.addAll(attrList);
				// remove duplicate attributes
				// int size = attrList.size();
				// for (int l = 0; l < size; l++) {
				// ServiceLocationAttribute servicelocationattribute =
				// (ServiceLocationAttribute)attrList.elementAt(l);
				// if (results.size() >= maxResults)
				// break;
				// }

			}
		}
	}

	public Object nextElement() {
		getNext();
		if (iList.size() == 0) { return null; }
		iTotalResults++;
		return iList.remove(0);
	}

};
