/**
 * SLPAttributeReplyMessage.java
 *
 * (C) Copyright IBM Corp. 2005, 2006
 *
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE COMMON PUBLIC LICENSE 
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE 
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Common Public License from
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * @author: Roberto Pineiro, IBM, roberto.pineiro@us.ibm.com  
 * @author: Chung-hao Tan, IBM ,chungtan@us.ibm.com
 * 
 * Change History
 * Flag       Date        Prog         Description
 *------------------------------------------------------------------------------- 
 * 1516246    2006-07-22  lupusalex    Integrate SLP client code
 */

package org.sblim.slp.internal;

import java.io.*;
import java.util.*;

import org.sblim.slp.ServiceLocationException;

public class SLPAttributeReplyMessage implements ServiceLocationMessage {

	private ServiceLocationHeader iHeader;

	private Vector iAttributeList = new Vector();

	private SLPAttributeReplyMessage() {}

	SLPAttributeReplyMessage(SLPHeaderV2 pHeader, DataInputStream pStream)
			throws ServiceLocationException, IOException {

		if (pHeader.getFunctionCode() != 7) throw new ServiceLocationException(
				ServiceLocationException.NETWORK_ERROR);

		iHeader = pHeader;

		if (pHeader.getErrorCode() != 0 || pHeader.isOverflow()) return;

		pHeader.parseAttributeVector(iAttributeList, pStream);

		// TODO validate signature
		pHeader.parseSignatureAndValidateAttributes(new Vector(), pStream);

		pHeader.setNumberOfReplies(iAttributeList.size());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.slp.ServiceLocationMessage#getErrorCode()
	 */
	public short getErrorCode() {
		return iHeader.getErrorCode();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.slp.ServiceLocationMessage#getHeader()
	 */
	public ServiceLocationHeader getHeader() {
		return iHeader;
	}

	/**
	 * Getter for attributeList
	 * 
	 * @return Returns the attributeList.
	 */
	public Vector getAttributeList() {
		return iAttributeList;
	}
}
