/**
 * SLPAttributeRequestMessage.java
 *
 * (C) Copyright IBM Corp. 2005, 2006
 *
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE COMMON PUBLIC LICENSE 
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE 
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Common Public License from
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * @author: Roberto Pineiro, IBM, roberto.pineiro@us.ibm.com  
 * @author: Chung-hao Tan, IBM ,chungtan@us.ibm.com
 * 
 * Change History
 * Flag       Date        Prog         Description
 *------------------------------------------------------------------------------- 
 * 1516246    2006-07-22  lupusalex    Integrate SLP client code
 */

package org.sblim.slp.internal;

import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.IOException;
import java.util.Hashtable;
import java.util.Locale;
import java.util.Vector;

import org.sblim.slp.ServiceLocationException;
import org.sblim.slp.ServiceType;
import org.sblim.slp.ServiceURL;

class SLPAttributeRequestMessage implements ServiceLocationMessage {

	private ServiceLocationHeader iHeader;

	private ServiceURL iUrl = null;

	private String iServiceType = null;

	private Vector iTags = new Vector();

	private String iSpi = "";

	protected SLPAttributeRequestMessage() {}

	SLPAttributeRequestMessage(ServiceLocationHeader pHeader, DataInputStream pStream)
			throws ServiceLocationException, IOException {
		if (pHeader.getFunctionCode() != 6) throw new ServiceLocationException(
				ServiceLocationException.NETWORK_ERROR);

		iHeader = pHeader;

		SLPServerHeaderV2 headerv2 = (SLPServerHeaderV2) pHeader;

		StringBuffer sb = new StringBuffer();

		headerv2.parsePreviousResponders(pStream);
		headerv2.getString(sb, pStream);
		String serviceurl = sb.toString();
		try {
			iUrl = new ServiceURL(serviceurl, 0);
			iServiceType = null;
		} catch (IllegalArgumentException e) {
			ServiceType servicetype = new ServiceType(serviceurl);
			iServiceType = servicetype.toString();
			iUrl = null;
		}
		headerv2.parseScopes(pStream);

		headerv2.getString(sb, pStream);
		iTags = Utils.parseCommaSeparatedList(sb.toString(), true);
		Utils.unescapeTags(iTags);
		headerv2.getString(sb, pStream);
		iSpi = sb.toString();
	}

	SLPAttributeRequestMessage(Locale pLocale, ServiceType pServiceType, Vector pScopes,
			Vector pAttributeIds) throws ServiceLocationException {
		initialize(pLocale, pServiceType.toString(), pScopes, pAttributeIds);
	}

	SLPAttributeRequestMessage(Locale pLocale, ServiceURL pServiceUrl, Vector pScopes,
			Vector pAttributeIds) throws ServiceLocationException {
		initialize(pLocale, pServiceUrl.toString(), pScopes, pAttributeIds);
	}

	private void initialize(Locale pLocale, String pService, Vector pScopes, Vector pAttributeIds)
			throws ServiceLocationException {
		iHeader = new SLPHeaderV2(6, false, pLocale);
		SLPHeaderV2 slpheaderv2 = (SLPHeaderV2) iHeader;
		slpheaderv2.setScopes((Vector) pScopes.clone());
		slpheaderv2.setPreviousResponse(new Vector());

		ByteArrayOutputStream tmpBuff = new ByteArrayOutputStream();
		slpheaderv2.putString(pService, tmpBuff);

		Utils.escapeScopeStrings(pScopes);
		slpheaderv2.serializeCommaSeparatedList(pScopes, tmpBuff);

		Utils.escapeTags(pAttributeIds);
		slpheaderv2.serializeCommaSeparatedList(pAttributeIds, tmpBuff);
		String spi = "";

		// TODO include SPI
		slpheaderv2.putString(spi, tmpBuff);
		slpheaderv2.setPayload(tmpBuff.toByteArray());
	}

	ServiceLocationMessage makeReply(Vector pAttributeIds, Hashtable pAuth)
			throws ServiceLocationException {
		SLPServerHeaderV2 header = ((SLPServerHeaderV2) getHeader()).makeReplyHeader();
		header.setNumberOfReplies(pAttributeIds.size());

		// TODO

		ByteArrayOutputStream buf = new ByteArrayOutputStream();
		// TODO add auth
		header.serializeAttributeVector(pAttributeIds, buf, false);
		header.setPayload(buf.toByteArray());

		return header;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.slp.ServiceLocationMessage#getErrorCode()
	 */
	public short getErrorCode() {
		return iHeader.getErrorCode();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.slp.ServiceLocationMessage#getHeader()
	 */
	public ServiceLocationHeader getHeader() {
		return iHeader;
	}

	/**
	 * Getter for spi
	 * 
	 * @return Returns the spi.
	 */
	public String getSpi() {
		return iSpi;
	}

	/**
	 * Getter for serviceType
	 * 
	 * @return Returns the serviceType.
	 */
	public String getServiceType() {
		return iServiceType;
	}

	/**
	 * Getter for tags
	 * 
	 * @return Returns the tags.
	 */
	public Vector getTags() {
		return iTags;
	}

	/**
	 * Getter for url
	 * 
	 * @return Returns the url.
	 */
	public ServiceURL getUrl() {
		return iUrl;
	}
}
