/**
 * SLPDAAdvert.java
 *
 * (C) Copyright IBM Corp. 2005, 2006
 *
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE COMMON PUBLIC LICENSE 
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE 
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Common Public License from
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * @author: Roberto Pineiro, IBM, roberto.pineiro@us.ibm.com  
 * @author: Chung-hao Tan, IBM ,chungtan@us.ibm.com
 * 
 * Change History
 * Flag       Date        Prog         Description
 *------------------------------------------------------------------------------- 
 * 1516246    2006-07-22  lupusalex    Integrate SLP client code
 * 1535793    2006-09-14  lupusalex    Fix&Integrate CIM&SLP configuration classes
 */

package org.sblim.slp.internal;

import java.io.DataInputStream;
import java.io.IOException;
import java.util.Vector;

import org.sblim.slp.ServiceLocationException;
import org.sblim.slp.ServiceType;
import org.sblim.slp.ServiceURL;

class SLPDAAdvert implements ServiceLocationMessage {

	private Vector iAttributes = new Vector();

	private ServiceLocationHeader iHeader;

	private String iSpis = null;

	private long iTimestamp = 0;

	private ServiceURL iUrl = null;

	SLPDAAdvert(ServiceLocationHeader pHeader, DataInputStream pStream)
			throws ServiceLocationException, IOException {

		if (pHeader.getFunctionCode() != 11) throw new ServiceLocationException(
				ServiceLocationException.NETWORK_ERROR);
		iHeader = pHeader;

		SLPHeaderV2 slpheaderv2 = (SLPHeaderV2) pHeader;

		pHeader.incrementNumberOfBytes(4);
		iTimestamp = ((pStream.read() & 0xff) << 24) + ((pStream.read() & 0xff) << 16)
				+ ((pStream.read() & 0xff) << 8) + ((pStream.read() & 0xff));

		StringBuffer sb = new StringBuffer();
		byte _url[] = slpheaderv2.getString(sb, pStream);

		int urlLifetime = getDaUrlLifetime();

		String serviceurl = sb.toString();
		byte _scopeList[] = slpheaderv2.getString(sb, pStream);
		slpheaderv2.setScopes(Utils.parseCommaSeparatedList(sb.toString(), true));
		Utils.unescapeScopeStrings(slpheaderv2.getScopes());

		// TODO validate scopes!

		byte _attributes[] = slpheaderv2.parseAttributeVector(iAttributes, pStream);
		byte _spis[] = slpheaderv2.getString(sb, pStream);
		iSpis = sb.toString();

		Vector vect = new Vector();
		slpheaderv2.parseSignatureAndValidateAttributes(vect, pStream);

		// TODO Get the minumum lifetime from the authorization block, otherwise
		// use the default from the config

		try {
			iUrl = new ServiceURL(serviceurl, urlLifetime);
		} catch (IllegalArgumentException e) {
			throw new ServiceLocationException(ServiceLocationException.PARSE_ERROR);
		}
		ServiceType servicetype = iUrl.getServiceType();

		if (!servicetype.equals(SLPDefaults.DA_SERVICE_TYPE)) throw new ServiceLocationException(
				ServiceLocationException.PARSE_ERROR);

		slpheaderv2.setNumberOfReplies(1);
	}

	/**
	 * Getter for attributes
	 * 
	 * @return Returns the attributes.
	 */
	public Vector getAttributes() {
		return iAttributes;
	}

	private int getDaUrlLifetime() {
		// SLPConfig slpconfig = SLPConfig.getSLPConfig();
		int interval = SLPConfig.getGlobalSLPConfig().getActiveDiscoveryInterval();
		int granularity = SLPConfig.getGlobalSLPConfig().getActiveDiscoveryGranularity();
		if (interval <= 0) { return 65535; }
		int k = interval + granularity;
		return k <= 65535 ? k : 65535;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.slp.ServiceLocationMessage#getErrorCode()
	 */
	public short getErrorCode() {
		return iHeader.getErrorCode();
	}

	public ServiceLocationHeader getHeader() {
		return iHeader;
	}

	/**
	 * Getter for spis
	 * 
	 * @return Returns the spis.
	 */
	public String getSpis() {
		return iSpis;
	}

	/**
	 * Getter for timestamp
	 * 
	 * @return Returns the timestamp.
	 */
	public long getTimestamp() {
		return iTimestamp;
	}

	/**
	 * Getter for url
	 * 
	 * @return Returns the url.
	 */
	public ServiceURL getUrl() {
		return iUrl;
	}

	public boolean isGoingDown() {
		return iTimestamp == 0L;
	}

	public boolean isUnsolicited() {
		return iHeader.getXid() == 0;
	}

	public void setIsUnsolicited(boolean flag) {
	// TODO
	}

}
