/**
 * SLPDatagramServer.java
 *
 * (C) Copyright IBM Corp. 2005, 2006
 *
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE COMMON PUBLIC LICENSE 
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE 
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Common Public License from
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * @author: Roberto Pineiro, IBM, roberto.pineiro@us.ibm.com  
 * @author: Chung-hao Tan, IBM ,chungtan@us.ibm.com
 * 
 * Change History
 * Flag       Date        Prog         Description
 *------------------------------------------------------------------------------- 
 * 1516246    2006-07-22  lupusalex    Integrate SLP client code
 * 1535793    2006-09-14  lupusalex    Fix&Integrate CIM&SLP configuration classes
 */

package org.sblim.slp.internal;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.MulticastSocket;
import java.net.SocketException;

/**
 */
public class SLPDatagramServer extends Thread {

	private DatagramSocket iDatagramSocket;

	private InetAddress iInetaddress;

	private int iPackageSize;

	private static DatagramSocket cBoadcastSocket;

	public SLPDatagramServer(InetAddress pInetAddress) {
		setDaemon(true);
		iPackageSize = SLPConfig.getGlobalSLPConfig().getMTU();

		iDatagramSocket = getMulticastSocket(pInetAddress);
		this.iInetaddress = pInetAddress;

		if (iDatagramSocket instanceof MulticastSocket) {
			MulticastSocket multicastsocket = (MulticastSocket) iDatagramSocket;
			try {
				multicastsocket.joinGroup(SLPConfig.getMulticastAddress());
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	private DatagramSocket getMulticastSocket(InetAddress pInetAddress) {
		DatagramSocket socket = null;
		if (SLPConfig.getGlobalSLPConfig().isBroadcastOnly()) {
			try {
				if (cBoadcastSocket != null) socket = cBoadcastSocket;
				else {
					socket = new DatagramSocket(SLPDefaults.SLP_PORT, SLPConfig
							.getBroadcastAddress());
					cBoadcastSocket = socket;
				}
			} catch (SocketException e) {
				e.printStackTrace();
			}
		} else {
			MulticastSocket multicastsocket = null;
			try {
				multicastsocket = new MulticastSocket(SLPDefaults.SLP_PORT);
				multicastsocket.setTimeToLive(SLPConfig.getGlobalSLPConfig().getMulticastRadius());
				multicastsocket.setInterface(pInetAddress);
			} catch (IOException e) {
				e.printStackTrace();
			}
			socket = multicastsocket;
		}
		return socket;
	}

	public void run() {
		// boolean flag = true;
		String s = "Multicast";
		setName("SLP " + (SLPConfig.getGlobalSLPConfig().isBroadcastOnly() ? "Broadcast" + s : s)
				+ " Datagram Listener:" + iDatagramSocket.getLocalAddress() + "/"
				+ iDatagramSocket.getLocalPort());
		do {
			byte buffer[] = new byte[iPackageSize];
			DatagramPacket datagrampacket = new DatagramPacket(buffer, iPackageSize);
			try {
				iDatagramSocket.receive(datagrampacket);
				ThreadPool.getInstance().execute(
						new SLPRequestHandler(datagrampacket, iInetaddress));
			} catch (IOException ioexception) {
				// flag = false;
				iDatagramSocket.close();
				try {
					iDatagramSocket = getMulticastSocket(iInetaddress);
					if (iDatagramSocket instanceof MulticastSocket) {
						MulticastSocket multicastsocket = (MulticastSocket) iDatagramSocket;
						multicastsocket.joinGroup(SLPConfig.getMulticastAddress());
					}
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		} while (true);
	}
}
