/**
 * SLPSAAdvert.java
 *
 * (C) Copyright IBM Corp. 2005, 2006
 *
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE COMMON PUBLIC LICENSE 
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE 
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Common Public License from
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * @author: Roberto Pineiro, IBM, roberto.pineiro@us.ibm.com  
 * @author: Chung-hao Tan, IBM ,chungtan@us.ibm.com
 * 
 * Change History
 * Flag       Date        Prog         Description
 *------------------------------------------------------------------------------- 
 * 1516246    2006-07-22  lupusalex    Integrate SLP client code
 */

package org.sblim.slp.internal;

import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.IOException;
import java.util.Locale;
import java.util.Vector;

import org.sblim.slp.ServiceLocationException;
import org.sblim.slp.ServiceType;
import org.sblim.slp.ServiceURL;

class SLPSAAdvert implements ServiceLocationMessage {

	private ServiceLocationHeader iHeader;

	private ServiceURL iUrl = null;

	// private Hashtable iAuthBlock = null;

	private Vector iAttributes = new Vector();

	public SLPSAAdvert(int pVersion, short pXid, Locale pLocale, ServiceURL pServiceurl,
			Vector pScopes, Vector pAttributes) throws ServiceLocationException {

		iHeader = new SLPServerHeaderV2();

		iHeader.setFunctionCode(ServiceLocationHeader.SAADVERT);
		iHeader.setXid(pXid);
		iHeader.setLocale(pLocale);

		SLPServerHeaderV2 slpserverheaderv2 = (SLPServerHeaderV2) iHeader;
		ServiceType servicetype = pServiceurl.getServiceType();
		if (!servicetype.equals(SLPDefaults.SA_SERVICE_TYPE)) throw new ServiceLocationException(
				ServiceLocationException.PARSE_ERROR);

		// TODO validate scopes

		slpserverheaderv2.setScopes((Vector) pScopes.clone());

		Utils.escapeScopeStrings(pScopes);
		ByteArrayOutputStream tmpBuffer = new ByteArrayOutputStream();

		byte authBlockSize = 0;

		String s = pServiceurl.toString();

		slpserverheaderv2.putString(s, tmpBuffer);
		slpserverheaderv2.serializeCommaSeparatedList(pScopes, tmpBuffer);
		slpserverheaderv2.serializeAttributeVector(pAttributes, tmpBuffer, false);

		tmpBuffer.write(authBlockSize);
		slpserverheaderv2.incrementNumberOfBytes(1);

		// TODO add authBlock information

		slpserverheaderv2.setPayload(tmpBuffer.toByteArray());
	}

	public SLPSAAdvert(SLPHeaderV2 pHeader, DataInputStream pStream)
			throws ServiceLocationException, IOException {

		if (pHeader.getFunctionCode() != 11) throw new ServiceLocationException(
				ServiceLocationException.NETWORK_ERROR);

		iHeader = pHeader;

		StringBuffer sb = new StringBuffer();
		byte _URL[] = pHeader.getString(sb, pStream);

		try {
			iUrl = new ServiceURL(sb.toString(), 0);
		} catch (IllegalArgumentException e) {
			throw new ServiceLocationException(ServiceLocationException.PARSE_ERROR);
		}

		ServiceType servicetype = iUrl.getServiceType();

		if (!servicetype.equals(SLPDefaults.SA_SERVICE_TYPE)) { throw new ServiceLocationException(
				ServiceLocationException.PARSE_ERROR); }

		byte _scopeList[] = pHeader.getString(sb, pStream);

		pHeader.setScopes(Utils.parseCommaSeparatedList(sb.toString(), true));
		Utils.unescapeScopeStrings(pHeader.getScopes());

		// TODO validate scopes!

		byte _attributes[] = pHeader.parseAttributeVector(iAttributes, pStream);

		Vector elementsValue = new Vector();
		// TODO pass the URL, attributes, and scopes for the validation

		pHeader.parseSignatureAndValidateAttributes(elementsValue, pStream);

		pHeader.setNumberOfReplies(1);
		return;

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.slp.ServiceLocationMessage#getErrorCode()
	 */
	public short getErrorCode() {
		return iHeader.getErrorCode();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.slp.ServiceLocationMessage#getHeader()
	 */
	public ServiceLocationHeader getHeader() {
		return iHeader;
	}

	/**
	 * Getter for attributes
	 * 
	 * @return Returns the attributes.
	 */
	public Vector getAttributes() {
		return iAttributes;
	}

	/**
	 * Getter for url
	 * 
	 * @return Returns the url.
	 */
	public ServiceURL getUrl() {
		return iUrl;
	}
}
