/**
 * SLPServiceDeregistration.java
 *
 * (C) Copyright IBM Corp. 2005, 2006
 *
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE COMMON PUBLIC LICENSE 
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE 
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Common Public License from
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * @author: Roberto Pineiro, IBM, roberto.pineiro@us.ibm.com  
 * @author: Chung-hao Tan, IBM ,chungtan@us.ibm.com
 * 
 * Change History
 * Flag       Date        Prog         Description
 *------------------------------------------------------------------------------- 
 * 1516246    2006-07-22  lupusalex    Integrate SLP client code
 */

package org.sblim.slp.internal;

import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.IOException;
import java.util.Hashtable;
import java.util.Locale;
import java.util.Vector;

import org.sblim.slp.ServiceLocationException;
import org.sblim.slp.ServiceURL;

public class SLPServiceDeregistration implements ServiceLocationMessage {

	private ServiceLocationHeader iHeader;

	private ServiceURL iUrl;

	private Hashtable iUrlSignature;

	private Vector iTags;

	public SLPServiceDeregistration(Locale pLocale, ServiceURL pServiceUrl, Vector pScopes,
			Vector pTagList) throws ServiceLocationException {

		SLPHeaderV2 slpheaderv2 = new SLPHeaderV2(4, false, pLocale);
		iHeader = slpheaderv2;

		slpheaderv2.setScopes((Vector) pScopes.clone());
		if (pTagList != null) Utils.escapeTags(pTagList);
		else pTagList = new Vector();
		ByteArrayOutputStream tmpBuff = new ByteArrayOutputStream();

		Utils.escapeScopeStrings(pScopes);
		slpheaderv2.serializeCommaSeparatedList(pScopes, tmpBuff);
		slpheaderv2.serializeServiceURL(pServiceUrl, tmpBuff, false);
		slpheaderv2.serializeCommaSeparatedList(pTagList, tmpBuff);
		slpheaderv2.setPayload(tmpBuff.toByteArray());
	}

	public SLPServiceDeregistration(ServiceLocationHeader pHeader, DataInputStream pStream)
			throws ServiceLocationException, IOException {
		if (pHeader.getFunctionCode() != 4) throw new ServiceLocationException(
				ServiceLocationException.NETWORK_ERROR);

		iHeader = pHeader;
		iUrl = null;
		iUrlSignature = null;
		iTags = null;

		SLPServerHeaderV2 slpserverheaderv2 = (SLPServerHeaderV2) pHeader;

		StringBuffer sb = new StringBuffer();
		slpserverheaderv2.parseScopes(pStream);
		// Hashtable hashtable = new Hashtable();

		iUrl = slpserverheaderv2.parseServiceURL(pStream, (short) 3);

		// TODO URLSignature
		slpserverheaderv2.getString(sb, pStream);

		iTags = Utils.parseCommaSeparatedList(sb.toString(), true);
		if (iTags.size() <= 0) iTags = null;
		else Utils.unescapeTags(iTags);
	}

	public ServiceLocationMessage makeReply() {
		SLPServerHeaderV2 slpserverheaderv2 = ((SLPServerHeaderV2) iHeader).makeReplyHeader();
		return slpserverheaderv2;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.slp.ServiceLocationMessage#getErrorCode()
	 */
	public short getErrorCode() {
		// TODO Auto-generated method stub
		return iHeader.getErrorCode();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.slp.ServiceLocationMessage#getHeader()
	 */
	public ServiceLocationHeader getHeader() {
		// TODO Auto-generated method stub
		return iHeader;
	}

	/**
	 * Getter for tags
	 * 
	 * @return Returns the tags.
	 */
	public Vector getTags() {
		return iTags;
	}

	/**
	 * Getter for url
	 * 
	 * @return Returns the url.
	 */
	public ServiceURL getUrl() {
		return iUrl;
	}

	/**
	 * Getter for urlSignature
	 * 
	 * @return Returns the urlSignature.
	 */
	public Hashtable getUrlSignature() {
		return iUrlSignature;
	}
}
