/**
 * SLPServiceRegistration.java
 *
 * (C) Copyright IBM Corp. 2005, 2006
 *
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE COMMON PUBLIC LICENSE 
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE 
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Common Public License from
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * @author: Roberto Pineiro, IBM, roberto.pineiro@us.ibm.com  
 * @author: Chung-hao Tan, IBM ,chungtan@us.ibm.com
 * 
 * Change History
 * Flag       Date        Prog         Description
 *------------------------------------------------------------------------------- 
 * 1516246    2006-07-22  lupusalex    Integrate SLP client code
 */

package org.sblim.slp.internal;

import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.IOException;
import java.util.*;

import org.sblim.slp.ServiceLocationAttribute;
import org.sblim.slp.ServiceLocationException;
import org.sblim.slp.ServiceType;
import org.sblim.slp.ServiceURL;

public class SLPServiceRegistration implements ServiceLocationMessage {

	private ServiceLocationHeader iHeader;

	private ServiceURL iUrl;

	private String iServiceType = "";

	private Vector iAttributes = new Vector();

	private Hashtable iUrlSignature;

	public SLPServiceRegistration(boolean pFresh, Locale pLocale, ServiceURL pServiceUrl,
			Vector pScopes, Vector pAttributes) throws ServiceLocationException {

		iUrl = pServiceUrl;

		Vector attributeList = new Vector();
		int size = pAttributes.size();
		for (int i = 0; i < size; i++) {
			Object obj = pAttributes.elementAt(i);
			if (!(obj instanceof ServiceLocationAttribute)) throw new IllegalArgumentException();
			// ServiceLocationAttribute servicelocationattribute =
			// (ServiceLocationAttribute) obj;
			// TODO remove duplicates attributes
		}
		attributeList = pAttributes;

		SLPHeaderV2 slpheaderv2 = new SLPHeaderV2(3, pFresh, pLocale);
		iHeader = slpheaderv2;

		slpheaderv2.setScopes((Vector) pScopes.clone());
		ByteArrayOutputStream tmpBuf = new ByteArrayOutputStream();
		slpheaderv2.serializeServiceURL(pServiceUrl, tmpBuf, false);

		ServiceType servicetype = pServiceUrl.getServiceType();
		slpheaderv2.putString(servicetype.toString(), tmpBuf);
		iServiceType = servicetype.toString();

		Utils.escapeScopeStrings(pScopes);
		slpheaderv2.serializeCommaSeparatedList(pScopes, tmpBuf);

		slpheaderv2.serializeAttributeVector(attributeList, tmpBuf, true);
		slpheaderv2.setPayload(tmpBuf.toByteArray());
	}

	public SLPServiceRegistration(ServiceLocationHeader pHeader, DataInputStream pStream)
			throws ServiceLocationException, IOException {

		if (pHeader.getFunctionCode() != 3) throw new ServiceLocationException(
				ServiceLocationException.NETWORK_ERROR);

		iHeader = pHeader;
		SLPServerHeaderV2 slpserverheaderv2 = (SLPServerHeaderV2) getHeader();
		StringBuffer sb = new StringBuffer();

		iUrl = slpserverheaderv2.parseServiceURL(pStream,
				ServiceLocationException.INVALID_REGISTRATION);

		slpserverheaderv2.getString(sb, pStream);
		ServiceType servicetype = new ServiceType(sb.toString());
		iServiceType = servicetype.toString();

		if (!servicetype.isServiceURL() && !servicetype.equals(iUrl.getServiceType())) iUrl
				.setServiceType(servicetype);
		slpserverheaderv2.parseScopes(pStream);
		slpserverheaderv2.parseAuthenticatedAttributeVector(iAttributes, pStream, false);

	}

	ServiceLocationMessage makeReply(boolean flag) {
		SLPServerHeaderV2 slpserverheaderv2 = ((SLPServerHeaderV2) iHeader).makeReplyHeader();
		return slpserverheaderv2;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.slp.ServiceLocationMessage#getErrorCode()
	 */
	public short getErrorCode() {
		// TODO Auto-generated method stub
		return iHeader.getErrorCode();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.slp.ServiceLocationMessage#getHeader()
	 */
	public ServiceLocationHeader getHeader() {
		// TODO Auto-generated method stub
		return iHeader;
	}

	/**
	 * Getter for attributes
	 * 
	 * @return Returns the attributes.
	 */
	public Vector getAttributes() {
		return iAttributes;
	}

	/**
	 * Getter for serviceType
	 * 
	 * @return Returns the serviceType.
	 */
	public String getServiceType() {
		return iServiceType;
	}

	/**
	 * Getter for url
	 * 
	 * @return Returns the url.
	 */
	public ServiceURL getUrl() {
		return iUrl;
	}

	/**
	 * Getter for urlSignature
	 * 
	 * @return Returns the urlSignature.
	 */
	public Hashtable getUrlSignature() {
		return iUrlSignature;
	}
}
