/**
 * SLPServiceReplyMessage.java
 *
 * (C) Copyright IBM Corp. 2005, 2006
 *
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE COMMON PUBLIC LICENSE 
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE 
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Common Public License from
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * @author: Roberto Pineiro, IBM, roberto.pineiro@us.ibm.com  
 * @author: Chung-hao Tan, IBM ,chungtan@us.ibm.com
 * 
 * Change History
 * Flag       Date        Prog         Description
 *------------------------------------------------------------------------------- 
 * 1516246    2006-07-22  lupusalex    Integrate SLP client code
 */

package org.sblim.slp.internal;

import java.io.DataInputStream;
import java.io.IOException;
import java.util.Vector;

import org.sblim.slp.ServiceLocationException;
import org.sblim.slp.ServiceURL;

class SLPServiceReplyMessage implements ServiceLocationMessage {

	private Vector iServiceURLs = new Vector();

	private ServiceLocationHeader iHeader = null;

	public SLPServiceReplyMessage(SLPHeaderV2 pHeader, DataInputStream pStream)
			throws ServiceLocationException, IOException {
		iHeader = null;
		if (pHeader.getFunctionCode() != 2) { throw new ServiceLocationException(
				ServiceLocationException.NETWORK_ERROR); }
		iHeader = pHeader;

		iServiceURLs = new Vector();

		if (pHeader.getErrorCode() == 0) {
			int size = pHeader.getInt(pStream);
			for (int i = 0; i < size; i++) {
				ServiceURL serviceurl = pHeader.parseServiceURL(pStream,
						ServiceLocationException.PARSE_ERROR);
				iServiceURLs.addElement(serviceurl);
				// verify URL using authblock
			}
			pHeader.setNumberOfReplies(iServiceURLs.size());
		}
	}

	public ServiceLocationHeader getHeader() {
		return iHeader;
	}

	public short getErrorCode() {
		return iHeader.getErrorCode();
	}

	/**
	 * Getter for serviceURLs
	 * 
	 * @return Returns the serviceURLs.
	 */
	public Vector getServiceURLs() {
		return iServiceURLs;
	}
}
