/**
 * SLPServiceRequestMessage.java
 *
 * (C) Copyright IBM Corp. 2005, 2006
 *
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE COMMON PUBLIC LICENSE 
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE 
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Common Public License from
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * @author: Roberto Pineiro, IBM, roberto.pineiro@us.ibm.com  
 * @author: Chung-hao Tan, IBM ,chungtan@us.ibm.com
 * 
 * Change History
 * Flag       Date        Prog         Description
 *------------------------------------------------------------------------------- 
 * 1516246    2006-07-22  lupusalex    Integrate SLP client code
 */

package org.sblim.slp.internal;

import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Locale;
import java.util.Vector;

import org.sblim.slp.ServiceLocationException;
import org.sblim.slp.ServiceType;
import org.sblim.slp.ServiceURL;

/**
 * 
 */
public class SLPServiceRequestMessage implements ServiceLocationMessage {

	private ServiceLocationHeader iHeader;

	private ServiceURL iUrl;

	private Vector iServiceURLs = new Vector();

	private ServiceType iServiceType = null;

	private String iQuery = "";

	private String iSpi = "";

	public SLPServiceRequestMessage(Locale pLocale, ServiceType pServiceType, Vector pScopes,
			String pQuery) throws ServiceLocationException {
		iServiceURLs = new Vector();

		SLPHeaderV2 slpheaderv2 = new SLPHeaderV2(1, false, pLocale);
		iHeader = slpheaderv2;

		slpheaderv2.setScopes((Vector) pScopes.clone());
		slpheaderv2.setPreviousResponse(new Vector());
		ByteArrayOutputStream tmpBuff = new ByteArrayOutputStream();
		Utils.escapeScopeStrings(pScopes);

		String spi = "";

		// TODO get SPI information
		slpheaderv2.putString(pServiceType.toString(), tmpBuff);
		slpheaderv2.serializeCommaSeparatedList(pScopes, tmpBuff);
		slpheaderv2.putString(pQuery, tmpBuff);
		slpheaderv2.putString(spi, tmpBuff);
		slpheaderv2.setPayload(tmpBuff.toByteArray());
	}

	public SLPServiceRequestMessage(ServiceLocationHeader pHeader, DataInputStream pStream)
			throws ServiceLocationException, IOException {

		if (pHeader.getFunctionCode() != 1) throw new ServiceLocationException(
				ServiceLocationException.NETWORK_ERROR);

		iHeader = pHeader;

		((SLPServerHeaderV2) pHeader).parsePreviousResponders(pStream);

		StringBuffer sb = new StringBuffer();
		pHeader.getString(sb, pStream);

		String serviceType = sb.toString();
		if (serviceType.length() <= 0) throw new ServiceLocationException(
				ServiceLocationException.PARSE_ERROR);

		// parse the serviceType
		this.iServiceType = new ServiceType(serviceType);

		pHeader.getString(sb, pStream);
		pHeader.setScopes(Utils.parseCommaSeparatedList(sb.toString(), true));
		if (pHeader.getScopes().size() <= 0) {
			// Scopes should be defined for directory-agent service requests
			// if (!serviceType.equals(SLPDefaults.SA_SERVICE_TYPE))
			// throw new
			// ServiceLocationException(ServiceLocationException.PARSE_ERROR);
		} else {
			// TODO validate scopes
		}
		pHeader.getString(sb, pStream);
		iQuery = sb.toString();

		pHeader.getString(sb, pStream);
		iSpi = sb.toString();
	}

	public ServiceLocationMessage makeReply(Vector pUrls) throws ServiceLocationException {

		SLPServerHeaderV2 rspHeader = ((SLPServerHeaderV2) getHeader()).makeReplyHeader();
		rspHeader.setNumberOfReplies(pUrls.size());
		ByteArrayOutputStream urlTempBuffer = new ByteArrayOutputStream();
		int totalUrls = pUrls.size();

		rspHeader.putInt(totalUrls, urlTempBuffer);

		Enumeration enumeration = pUrls.elements();
		for (int j = 0; enumeration.hasMoreElements(); j++) {
			ServiceURL serviceurl = (ServiceURL) enumeration.nextElement();
			Hashtable spiAuthBlock = null;

			if (rspHeader.serializeServiceURL(serviceurl, spiAuthBlock, urlTempBuffer, true)) continue;
			rspHeader.setOverflow(true);
			byte _urlTempBuffer[] = urlTempBuffer.toByteArray();
			urlTempBuffer.reset();
			ServiceLocationHeader.putInteger(j, urlTempBuffer);
			urlTempBuffer.write(_urlTempBuffer, 2, _urlTempBuffer.length - 2);
			break;
		}

		rspHeader.setPayload(urlTempBuffer.toByteArray());
		return rspHeader;
	}

	public short getErrorCode() {
		// TODO Auto-generated method stub
		return iHeader.getErrorCode();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.slp.ServiceLocationMessage#getHeader()
	 */
	public ServiceLocationHeader getHeader() {
		// TODO Auto-generated method stub
		return iHeader;
	}

	/**
	 * Getter for query
	 * 
	 * @return Returns the query.
	 */
	public String getQuery() {
		return iQuery;
	}

	/**
	 * Getter for serviceType
	 * 
	 * @return Returns the serviceType.
	 */
	public ServiceType getServiceType() {
		return iServiceType;
	}

	/**
	 * Getter for serviceURLs
	 * 
	 * @return Returns the serviceURLs.
	 */
	public Vector getServiceURLs() {
		return iServiceURLs;
	}

	/**
	 * Getter for spi
	 * 
	 * @return Returns the spi.
	 */
	public String getSpi() {
		return iSpi;
	}

	/**
	 * Getter for url
	 * 
	 * @return Returns the url.
	 */
	public ServiceURL getUrl() {
		return iUrl;
	}
}
