/**
 * SLPServiceTypeReplyMessage.java
 *
 * (C) Copyright IBM Corp. 2005, 2006
 *
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE COMMON PUBLIC LICENSE 
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE 
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Common Public License from
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * @author: Roberto Pineiro, IBM, roberto.pineiro@us.ibm.com  
 * @author: Chung-hao Tan, IBM ,chungtan@us.ibm.com
 * 
 * Change History
 * Flag       Date        Prog         Description
 *------------------------------------------------------------------------------- 
 * 1516246    2006-07-22  lupusalex    Integrate SLP client code
 */

package org.sblim.slp.internal;

import java.io.DataInputStream;
import java.io.IOException;
import java.util.Vector;

import org.sblim.slp.ServiceLocationException;
import org.sblim.slp.ServiceType;

public class SLPServiceTypeReplyMessage implements ServiceLocationMessage {

	private ServiceLocationHeader iHeader;

	private Vector iServiceTypes = new Vector();

	public SLPServiceTypeReplyMessage(SLPHeaderV2 pHeader, DataInputStream pStream)
			throws ServiceLocationException, IOException {

		if (pHeader.getFunctionCode() != 10) throw new ServiceLocationException(
				ServiceLocationException.NETWORK_ERROR);

		iHeader = pHeader;

		if (((ServiceLocationHeader) (pHeader)).getErrorCode() != 0) return;
		if (((ServiceLocationHeader) (pHeader)).isOverflow()) return;

		StringBuffer sb = new StringBuffer();

		pHeader.getString(sb, pStream);
		iServiceTypes = Utils.parseCommaSeparatedList(sb.toString(), true);

		int size = iServiceTypes.size();
		for (int i = 0; i < size; i++) {
			ServiceType servicetype = new ServiceType((String) iServiceTypes.elementAt(i));
			iServiceTypes.setElementAt(servicetype, i);
		}

		pHeader.setNumberOfReplies(iServiceTypes.size());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.slp.ServiceLocationMessage#getErrorCode()
	 */
	public short getErrorCode() {
		// TODO Auto-generated method stub
		return iHeader.getErrorCode();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.slp.ServiceLocationMessage#getHeader()
	 */
	public ServiceLocationHeader getHeader() {
		// TODO Auto-generated method stub
		return iHeader;
	}

	/**
	 * Getter for serviceTypes
	 * 
	 * @return Returns the serviceTypes.
	 */
	public Vector getServiceTypes() {
		return iServiceTypes;
	}
}
