/**
 * SLPServiceTypeRequestMessage.java
 *
 * (C) Copyright IBM Corp. 2005, 2006
 *
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE COMMON PUBLIC LICENSE 
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE 
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Common Public License from
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * @author: Roberto Pineiro, IBM, roberto.pineiro@us.ibm.com  
 * @author: Chung-hao Tan, IBM ,chungtan@us.ibm.com
 * 
 * Change History
 * Flag       Date        Prog         Description
 *------------------------------------------------------------------------------- 
 * 1516246    2006-07-22  lupusalex    Integrate SLP client code
 */

package org.sblim.slp.internal;

import java.io.*;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Locale;
import java.util.Vector;

import org.sblim.slp.ServiceLocationException;
import org.sblim.slp.ServiceURL;

class SLPServiceTypeRequestMessage implements ServiceLocationMessage {

	private ServiceLocationHeader iHeader;

	private Vector iServiceTypes = new Vector();

	private String iNamingAuthority;

	public SLPServiceTypeRequestMessage(ServiceLocationHeader pHeader, DataInputStream pStream)
			throws ServiceLocationException, IOException {

		if (pHeader.getFunctionCode() != 9) throw new ServiceLocationException(
				ServiceLocationException.NETWORK_ERROR);

		iHeader = pHeader;
		iNamingAuthority = "";

		SLPServerHeaderV2 headerv2 = (SLPServerHeaderV2) pHeader;

		// StringBuffer sb = new StringBuffer();
		headerv2.parsePreviousResponders(pStream);
		iNamingAuthority = parseNamingAuthority(headerv2, pStream, "UTF8");

		if (!iNamingAuthority.equalsIgnoreCase("iana")) {
			headerv2.parseScopes(pStream);
			return;
		}
		throw new ServiceLocationException(ServiceLocationException.PARSE_ERROR);
	}

	private String parseNamingAuthority(ServiceLocationHeader pSrvLocHeader,
			DataInputStream pStream, String pEncoding) throws ServiceLocationException, IOException {
		int i = 0;
		i = pSrvLocHeader.getInt(pStream);
		if (i == 0) return "";
		if (i == 65535) { return "*"; }
		byte _buff[] = new byte[i];
		pStream.readFully(_buff, 0, i);
		pSrvLocHeader.incrementNumberOfBytes(i);
		String na = ServiceLocationHeader.getBytesString(_buff, pEncoding).toLowerCase();
		// ServiceType.validateTypeComponent(na);
		return na;
	}

	public SLPServiceTypeRequestMessage(Locale pLocale, String pNamingAuthority, Vector pScopes)
			throws ServiceLocationException {

		iServiceTypes = new Vector();
		SLPHeaderV2 slpheaderv2 = new SLPHeaderV2(9, false, pLocale);

		iHeader = slpheaderv2;
		this.iNamingAuthority = pNamingAuthority;

		slpheaderv2.setScopes((Vector) pScopes.clone());
		String servicetypeLower = pNamingAuthority.toLowerCase();

		// if (!servicetypeLower.equals("*"))
		// ServiceType.validateTypeComponent(servicetypeLower);

		slpheaderv2.setPreviousResponse(new Vector());
		ByteArrayOutputStream tmpBuff = new ByteArrayOutputStream();

		if (servicetypeLower.length() <= 0) slpheaderv2.putInt(0, tmpBuff);
		else if (servicetypeLower.equals("*")) slpheaderv2.putInt(65535, tmpBuff);
		else slpheaderv2.putString(servicetypeLower, tmpBuff);

		Utils.escapeScopeStrings(slpheaderv2.getScopes());

		slpheaderv2.serializeCommaSeparatedList(slpheaderv2.getScopes(), tmpBuff);
		slpheaderv2.setPayload(tmpBuff.toByteArray());
	}

	public ServiceLocationMessage makeReply(Vector pUrls) throws ServiceLocationException {

		SLPServerHeaderV2 rspHeader = ((SLPServerHeaderV2) getHeader()).makeReplyHeader();
		rspHeader.setNumberOfReplies(pUrls.size());
		ByteArrayOutputStream urlTempBuffer = new ByteArrayOutputStream();
		int totalUrls = pUrls.size();

		rspHeader.putInt(totalUrls, urlTempBuffer);

		Enumeration enumeration = pUrls.elements();
		for (int j = 0; enumeration.hasMoreElements(); j++) {
			ServiceURL serviceurl = (ServiceURL) enumeration.nextElement();
			Hashtable spiAuthBlock = null;

			if (rspHeader.serializeServiceURL(serviceurl, spiAuthBlock, urlTempBuffer, true)) continue;
			rspHeader.setOverflow(true);
			byte _urlTempBuffer[] = urlTempBuffer.toByteArray();
			urlTempBuffer.reset();
			ServiceLocationHeader.putInteger(j, urlTempBuffer);
			urlTempBuffer.write(_urlTempBuffer, 2, _urlTempBuffer.length - 2);
			break;
		}

		rspHeader.setPayload(urlTempBuffer.toByteArray());
		return rspHeader;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.slp.ServiceLocationMessage#getErrorCode()
	 */
	public short getErrorCode() {
		// TODO Auto-generated method stub
		return iHeader.getErrorCode();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.slp.ServiceLocationMessage#getHeader()
	 */
	public ServiceLocationHeader getHeader() {
		// TODO Auto-generated method stub
		return iHeader;
	}

	/**
	 * Getter for namingAuthority
	 * 
	 * @return Returns the namingAuthority.
	 */
	public String getNamingAuthority() {
		return iNamingAuthority;
	}

	/**
	 * Getter for serviceTypes
	 * 
	 * @return Returns the serviceTypes.
	 */
	public Vector getServiceTypes() {
		return iServiceTypes;
	}
}
