/**
 * SLPStreamServer.java
 *
 * (C) Copyright IBM Corp. 2005, 2006
 *
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE COMMON PUBLIC LICENSE 
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE 
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Common Public License from
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * @author: Roberto Pineiro, IBM, roberto.pineiro@us.ibm.com  
 * @author: Chung-hao Tan, IBM ,chungtan@us.ibm.com
 * 
 * Change History
 * Flag       Date        Prog         Description
 *------------------------------------------------------------------------------- 
 * 1516246    2006-07-22  lupusalex    Integrate SLP client code
 * 1535793    2006-09-14  lupusalex    Fix&Integrate CIM&SLP configuration classes
 */

package org.sblim.slp.internal;

import java.io.IOException;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketException;

import org.sblim.slp.ServiceLocationException;

public class SLPStreamServer extends Thread {

	private boolean iAlive;

	// private SLPConfig iConfig;

	private ServerSocket iServerSocket;

	// private Vector iConnectionPool;

	private InetAddress iInetAddress;

	SLPStreamServer(InetAddress pInetaddress) throws ServiceLocationException {
		setDaemon(true);
		iAlive = true;
		int queueLength = SLPConfig.getGlobalSLPConfig().getServerSocketQueueLength();
		try {
			iServerSocket = new ServerSocket(427, queueLength, pInetaddress);
		} catch (IOException e) {
			throw new ServiceLocationException(ServiceLocationException.NETWORK_INIT_FAILED, e);
		}
	}

	public void startServer() {
		iAlive = true;
		super.start();
	}

	public synchronized void shutdownServer() {
		iAlive = false;
		try {
			if (iServerSocket != null) {
				ServerSocket oldServer = iServerSocket;
				iServerSocket = null;
				oldServer.close();
			}
		} catch (IOException e) {
			// TODO log this
		}
	}

	public void run() {
		setName("SLP Stream Listener:" + iServerSocket.getLocalSocketAddress() + "/"
				+ iServerSocket.getLocalPort());
		do {
			Socket clientSocket = null;
			try {
				if (iServerSocket != null) clientSocket = iServerSocket.accept();
				if (!iAlive) break;
				if (clientSocket != null) clientSocket.setSoTimeout(SLPConfig.getGlobalSLPConfig().getTCPTimeout());

				ThreadPool.getInstance().execute(new SLPRequestHandler(clientSocket, iInetAddress));
			} catch (SocketException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			}
		} while (true);
	}
}
