/**
 * ServiceLocationEnumeratorStreamed.java
 *
 * (C) Copyright IBM Corp. 2005, 2006
 *
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE COMMON PUBLIC LICENSE 
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE 
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Common Public License from
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * @author: Roberto Pineiro, IBM, roberto.pineiro@us.ibm.com  
 * @author: Chung-hao Tan, IBM ,chungtan@us.ibm.com
 * 
 * Change History
 * Flag       Date        Prog         Description
 *------------------------------------------------------------------------------- 
 * 1516246    2006-07-22  lupusalex    Integrate SLP client code
 * 1535793    2006-09-14  lupusalex    Fix&Integrate CIM&SLP configuration classes
 */

package org.sblim.slp.internal;

import java.io.ByteArrayInputStream;
import java.io.DataInputStream;
import java.io.IOException;
import java.io.InterruptedIOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.MulticastSocket;
import java.net.SocketException;
import java.util.Enumeration;
import java.util.Vector;

import org.sblim.slp.ServiceLocationEnumeration;
import org.sblim.slp.ServiceLocationException;

public class ServiceLocationEnumeratorStreamed implements ServiceLocationEnumeration {

	private boolean iHasMoreElements = true;

	private int iRetry = 0;

	private int iElapsedTime = 0;

	private Vector iResultSet = new Vector();

	private int iTotalResponses = 0;

	private Enumeration iEnumeration = null;

	private Vector iDAList;

	private ServiceLocationMessage iRequestMsg;

	private InetAddress iInetAddress;

	public ServiceLocationEnumeratorStreamed(Vector pDaList, ServiceLocationMessage pRequestMsg,
			InetAddress pInetaddress) {
		iDAList = pDaList;
		iRequestMsg = pRequestMsg;
		iInetAddress = pInetaddress;
	}

	public Object next() throws ServiceLocationException {
		return iEnumeration.nextElement();
	}

	public boolean hasMoreElements() throws ServiceLocationException {
		int[] multicastTimeouts = null;
		multicastTimeouts = SLPConfig.getGlobalSLPConfig().getMulticastTimeouts();
		if (iHasMoreElements && iResultSet.size() == 0 && iRetry < multicastTimeouts.length) {
			ServiceLocationException daException = null;
			ServiceLocationException saException = null;

			// TODO follow the RFC 2608 sec 12.3, multicast must be used in case
			// no DA is present

			if (iDAList != null && iDAList.size() > 0) {
				daException = MessageDispatcher.dispatchDAMsg(iDAList, iRequestMsg, iResultSet,
						SLPConfig.getGlobalSLPConfig().getMaximumResults());

				if (daException != null
						&& daException.getErrorCode() != ServiceLocationException.OK) {
					dispatchDatagramMsg(iRequestMsg, iResultSet, SLPConfig.getGlobalSLPConfig().getMulticastTimeouts(),
							SLPConfig.getGlobalSLPConfig().getMaximumResults(), iInetAddress, false);
				} else {
					iHasMoreElements = false;
				}
			} else {
				saException = dispatchDatagramMsg(iRequestMsg, iResultSet, multicastTimeouts,
						SLPConfig.getGlobalSLPConfig().getMaximumResults(), iInetAddress, false);
			}

			if (iResultSet.size() <= 0) {
				if (daException != null) {
					short errorCode = daException.getErrorCode();
					if (errorCode != ServiceLocationException.VERSION_NOT_SUPPORTED
							&& errorCode != ServiceLocationException.OK
							&& errorCode != ServiceLocationException.INTERNAL_ERROR
							&& errorCode != ServiceLocationException.OPTION_NOT_SUPPORTED
							&& errorCode != ServiceLocationException.REQUEST_NOT_SUPPORTED) throw daException;
				}
				if (saException != null) {
					short errorCode = saException.getErrorCode();
					if (errorCode != ServiceLocationException.VERSION_NOT_SUPPORTED
							&& errorCode != ServiceLocationException.OK
							&& errorCode != ServiceLocationException.INTERNAL_ERROR
							&& errorCode != ServiceLocationException.OPTION_NOT_SUPPORTED
							&& errorCode != ServiceLocationException.REQUEST_NOT_SUPPORTED) throw saException;
				}
			}
		}
		return iResultSet.size() > 0;
	}

	public Object nextElement() {
		if (iResultSet != null && iResultSet.size() > 0) return iResultSet.remove(0);
		return null;
	}

	private static int getNextTimeoutValue(int pElapsedTime, int pMaxTimeOut, int[] pMsTimeOutsList) {

		// int elapsedTime = (int)(currentMessageTime - initialTime);

		if (pMaxTimeOut < pElapsedTime) return -1;

		int acumulativeTime = 0;
		for (int i = 0; i < pMsTimeOutsList.length; i++) {

			acumulativeTime += pMsTimeOutsList[i];

			int tmpMaxTimeOut = acumulativeTime - pElapsedTime;
			if (pElapsedTime < acumulativeTime) {
				if (pMaxTimeOut - tmpMaxTimeOut < pElapsedTime) { return pMaxTimeOut - pElapsedTime; }
				return tmpMaxTimeOut;
			}
		}

		return -1;
	}

	public Vector dispatchMulticastMsg(InetAddress pInetaddress, DatagramSocket pDatagramsocket1,
			ServiceLocationMessage pRequest, int[] pMsTimeOutsList, int pMaxResults,
			boolean pGetFirstResult) throws ServiceLocationException {

		// TODO: Implement Logging
		// if (SLPConfig.logger != null) {
		// SLPConfig.logger.entry(Level.INFO,
		// ServiceLocationEnumeratorStreamed.class,
		// "ServiceLocationEnumeratorStreamed");
		// SLPConfig.logger.text(Level.INFO,
		// ServiceLocationEnumeratorStreamed.class,
		// "ServiceLocationEnumeratorStreamed",
		// "dispatchMulticastMsg("+
		// "inetaddress="+inetaddress+"\n"+
		// "datagramsocket="+datagramsocket+"\n" +
		// "request="+request+"\n"+
		// "msTimeOutsList="+msTimeOutsList+"\n"+
		// "maxResults="+maxResults+"\n"+
		// "getFirstResult="+getFirstResult+")");
		// }

		Vector results = new Vector();
		// int totalResponses = 0;
		// int retry = 0;
		int maxTimeOut = SLPConfig.getGlobalSLPConfig().getMulticastMaximumWait();

		long initialTime = System.currentTimeMillis();
		int mtu = SLPConfig.getGlobalSLPConfig().getMTU();
		try {
			byte _request[] = MessageDispatcher.getBytes(pRequest, true, false);
			pDatagramsocket1.send(new DatagramPacket(_request, _request.length, pInetaddress, 427));

			// retry++;
			long currentMessageTime = System.currentTimeMillis();
			while (iTotalResponses < pMaxResults) {

				byte tmpBuf[] = new byte[mtu];
				DatagramPacket datagrampacket = new DatagramPacket(tmpBuf, tmpBuf.length);

				int currentTimeoutvalue = getNextTimeoutValue((int) (currentMessageTime
						- initialTime + iElapsedTime), maxTimeOut, pMsTimeOutsList);

				// System.out.println("currentTimeout:"+currentTimeoutvalue);
				if (currentTimeoutvalue <= 0) break; // we are done

				pDatagramsocket1.setSoTimeout(currentTimeoutvalue);
				try {
					pDatagramsocket1.receive(datagrampacket);
				} catch (InterruptedIOException e) {
					if (results.size() > 0) break;

					int elapsed = (int) (System.currentTimeMillis() - initialTime) + iElapsedTime;
					if ((pGetFirstResult && iTotalResponses > 0) || elapsed > maxTimeOut
							|| iRetry >= pMsTimeOutsList.length) {

						// TODO: Implement Logging
						// if (SLPConfig.logger != null) {
						// SLPConfig.logger.text(Level.INFO,
						// ServiceLocationEnumeratorStreamed.class,
						// "dispatchMulticastMsg",
						// "totalResponses="+totalResponses+"\n"+
						// "elapsed="+elapsed+"\n" +
						// "request="+request+"\n"+
						// "maxTimeOut="+maxTimeOut+"\n"+
						// "retry="+retry+"\n"+
						// "msTimeOutsList.length="+msTimeOutsList.length);
						// }

						break;
					}
					// TODO: Implement Logging
					// if (SLPConfig.logger != null) {
					// SLPConfig.logger.text(Level.INFO,
					// ServiceLocationEnumeratorStreamed.class.getClass(),
					// "reattempting the operation request", "");
					// }
					pDatagramsocket1.send(new DatagramPacket(_request, _request.length,
							pInetaddress, 427));

					iRetry++;
					currentMessageTime = System.currentTimeMillis();
					continue;
				}

				DataInputStream dis = null;
				// TODO: Implement Logging
				// if (SLPConfig.logger != null)
				// dis = new DataInputStream(new DebugInputStream(new
				// ByteArrayInputStream(datagrampacket.getData()), System.out));
				// else
				dis = new DataInputStream(new ByteArrayInputStream(datagrampacket.getData()));

				InetAddress responderInetAddress = datagrampacket.getAddress();

				// TODO: Implement Logging
				// if (SLPConfig.logger != null) {
				// SLPConfig.logger.text(Level.INFO,
				// ServiceLocationEnumeratorStreamed.class,
				// "dispatchMulticastMsg","responderInetAddress="+responderInetAddress);
				// }

				ServiceLocationMessage response = null;

				try {
					response = MessageDispatcher.parseMsg(dis);
				} catch (ServiceLocationException e) {
					// TODO: Implement Logging
					// if (SLPConfig.logger != null) {
					// SLPConfig.logger.exception(Level.INFO,
					// ServiceLocationEnumeratorStreamed.class,
					// "dispatchMulticastMsg", e, "Error while parsing
					// message...");
					// }

					if (e.getErrorCode() == ServiceLocationException.PARSE_ERROR) {
						MessageDispatcher.addPreviousResponder(pRequest, responderInetAddress);
						continue;
					}
					if (e.getErrorCode() == ServiceLocationException.VERSION_NOT_SUPPORTED) {
						continue;
					}
					throw e;
				}

				if (pRequest.getHeader().getXid() != response.getHeader().getXid()
						|| !MessageDispatcher.addPreviousResponder(pRequest, responderInetAddress)) continue;

				ServiceLocationHeader rspHeader = response.getHeader();
				if (rspHeader.isOverflow()) {
					// TODO: Implement Logging
					// if (SLPConfig.logger != null) {
					// SLPConfig.logger.text(Level.INFO,
					// ServiceLocationEnumeratorStreamed.class,
					// "dispatchMulticastMsg", "message overflow");
					// }
					response = MessageDispatcher.dispatchStreamMsg(responderInetAddress, pRequest,
							false);
					if (response == null) {
						// TODO: Implement Logging
						// if (SLPConfig.logger != null) {
						// SLPConfig.logger.text(Level.INFO,
						// ServiceLocationEnumeratorStreamed.class,
						// "dispatchMulticastMsg", "no response received from
						// TPC/IP request");
						// }
						continue;
					}
					rspHeader = response.getHeader();
				}

				if (results.size() < pMaxResults) {
					// TODO: Implement Logging
					// if (SLPConfig.logger != null) {
					// SLPConfig.logger.text(Level.INFO,
					// ServiceLocationEnumeratorStreamed.class,
					// "dispatchMulticastMsg", "Adding responses to the list of
					// results");
					// }
					results.addElement(response);
				} else {
					// TODO: Implement Logging
					// if (SLPConfig.logger != null) {
					// SLPConfig.logger.text(Level.INFO,
					// ServiceLocationEnumeratorStreamed.class,
					// "dispatchMulticastMsg", "The number of received responses
					// exceed the maximum value of allowed response");
					// }
				}
				// TODO: Implement Logging
				// if (SLPConfig.logger != null) {
				// SLPConfig.logger.text(Level.INFO,
				// ServiceLocationEnumeratorStreamed.class,
				// "dispatchMulticastMsg", "responses received:
				// "+rspHeader.numReplies+"\nnew total
				// responses:"+totalResponses);
				// }
				iTotalResponses += rspHeader.getNumberOfReplies();
				if (pGetFirstResult) {
					// TODO: Implement Logging
					// if (SLPConfig.logger != null) {
					// SLPConfig.logger.text(Level.INFO,
					// ServiceLocationEnumeratorStreamed.class,
					// "dispatchMulticastMsg", "returning first result...");
					// }
					break;
				}
			}
		} catch (ServiceLocationException e) {
			// TODO: Implement Logging
			// if (SLPConfig.logger != null) {
			// SLPConfig.logger.exception(Level.INFO,
			// ServiceLocationEnumeratorStreamed.class, "dispatchMulticastMsg",
			// e);
			// }
			if (e.getErrorCode() == ServiceLocationException.PREVIOUS_RESPONDER_OVERFLOW) {
				// TODO: Implement Logging
				// if (SLPConfig.logger != null) {
				// SLPConfig.logger.text(Level.INFO,
				// ServiceLocationEnumeratorStreamed.class,
				// "dispatchMulticastMsg","Previous responder overflow");
				// SLPConfig.logger.exit(Level.INFO, MessageDispatcher.class,
				// "dispatchMulticastMsg",results);
				// }
				return results;
			}
			// TODO: Implement Logging
			// if (SLPConfig.logger != null)
			// SLPConfig.logger.text(Level.INFO,
			// ServiceLocationEnumeratorStreamed.class,
			// "dispatchMulticastMsg","propagating exception");
			throw e;
		} catch (IOException ioexception) {
			// TODO: Implement Logging
			// if (SLPConfig.logger != null) {
			// SLPConfig.logger.exception(Level.INFO,
			// ServiceLocationEnumeratorStreamed.class, "dispatchMulticastMsg",
			// ioexception);
			// }
			throw new ServiceLocationException(ServiceLocationException.NETWORK_ERROR, ioexception);
		} finally {
			iElapsedTime += System.currentTimeMillis() - initialTime;
			// TODO: Implement Logging
			// if (SLPConfig.logger != null) {
			// SLPConfig.logger.text(Level.INFO,
			// ServiceLocationEnumeratorStreamed.class,
			// "dispatchMulticastMsg", "new elapseTime="+elapsedTime);
			// }
		}
		// TODO: Implement Logging
		// if (SLPConfig.logger != null) {
		// SLPConfig.logger.exit(Level.INFO, MessageDispatcher.class,
		// "dispatchMulticastMsg",results);
		// }

		return results;
	}

	public ServiceLocationException dispatchDatagramMsg(ServiceLocationMessage pRequest,
			Vector pResponseList, int[] pTimeoutList, int pMaxResults, InetAddress pInetAddress,
			boolean pJustFirstResult) {

		DatagramSocket datagramsocket = null;

		int maxTimeOut = SLPConfig.getGlobalSLPConfig().getMulticastMaximumWait();

		if ((pJustFirstResult && iTotalResponses > 0) || iElapsedTime > maxTimeOut
				|| iRetry >= pTimeoutList.length) { return null; }

		boolean usingBroadcast = SLPConfig.getGlobalSLPConfig().isBroadcastOnly();
		if (usingBroadcast) pInetAddress = SLPConfig.getBroadcastAddress();

		try {
			// InetAddress localhost = SLPConfig.getLocalHost();
			if (usingBroadcast) {
				try {
					datagramsocket = new DatagramSocket();
				} catch (SocketException e) {
					datagramsocket = null;
					throw e;
				}
			} else {
				MulticastSocket multicastsocket = null;
				try {
					multicastsocket = new MulticastSocket();
					multicastsocket.setTimeToLive(SLPConfig.getGlobalSLPConfig().getMulticastRadius());
					// multicastsocket.setInterface(localhost);
				} catch (IOException e) {
					datagramsocket = null;
					throw e;
				}
				datagramsocket = multicastsocket;
			}

			pResponseList.addAll(dispatchMulticastMsg(pInetAddress, datagramsocket, pRequest,
					pTimeoutList, pMaxResults, pJustFirstResult));

			// datagramsocket.close();
			// datagramsocket = null;
		} catch (ServiceLocationException e) {
			if (e.getErrorCode() != ServiceLocationException.DA_BUSY) { return e; }
		} catch (Exception e) {
			return new ServiceLocationException(ServiceLocationException.INTERNAL_SYSTEM_ERROR, e);
		} finally {
			if (datagramsocket != null) datagramsocket.close();
		}
		return null;
	}

	public static Enumeration dispatchUAMsg(Vector pDAList, ServiceLocationMessage pRequestMsg,
			InetAddress pInetAddress) throws ServiceLocationException {

		return new ServiceLocationEnumeratorStreamed(pDAList, pRequestMsg, pInetAddress);
	}

}
