/**
 * ServiceRepository.java
 *
 * (C) Copyright IBM Corp. 2005, 2006
 *
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE COMMON PUBLIC LICENSE 
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE 
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Common Public License from
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * @author: Roberto Pineiro, IBM, roberto.pineiro@us.ibm.com  
 * @author: Chung-hao Tan, IBM ,chungtan@us.ibm.com
 * 
 * Change History
 * Flag       Date        Prog         Description
 *------------------------------------------------------------------------------- 
 * 1516246    2006-07-22  lupusalex    Integrate SLP client code
 */

package org.sblim.slp.internal;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Locale;
import java.util.Vector;

import org.sblim.slp.ServiceLocationException;
import org.sblim.slp.ServiceType;
import org.sblim.slp.ServiceURL;

public class ServiceRepository {

	class ServiceEntry {

		Vector iAttributes;

		Locale iLocale;

		Vector iScopes;

		ServiceURL iServiceurl;

		public ServiceEntry(ServiceURL pServiceUrl, Vector pAttributes, Vector pScopes,
				Locale pLocale) {
			iServiceurl = pServiceUrl;
			iAttributes = pAttributes;
			iScopes = pScopes;
			iLocale = pLocale;
		}

		public void merge(ServiceEntry that) {

			Enumeration enumeration = that.iScopes.elements();
			while (enumeration.hasMoreElements()) {
				String scope = (String) enumeration.nextElement();
				if (!iScopes.contains(scope)) {
					iScopes.add(scope);
				}
			}

			enumeration = that.iAttributes.elements();
			while (enumeration.hasMoreElements()) {
				String attribute = (String) enumeration.nextElement();
				if (!iAttributes.contains(attribute)) {
					iAttributes.add(attribute);
				}
			}
		}
	}

	private Hashtable iScopeLanguageType = new Hashtable();

	private Hashtable iUrlScopeLanguage = new Hashtable();

	public ServiceRepository() {
		super();
	}

	// public synchronized void deleteAttributes(ServiceURL pServiceUrl, Vector
	// pVector, Vector pVector1, Locale pLocale) {
	// }

	public synchronized void deregister(ServiceURL pServiceUrl, Vector pScopes)
			throws ServiceLocationException {
		ServiceEntry entry = findEntry(pServiceUrl, pScopes, null);
		if (entry == null) { throw new ServiceLocationException(
				ServiceLocationException.INVALID_UPDATE); }

		deregisterEntry(pServiceUrl, pScopes, null);
	}

	private void deregisterEntry(ServiceURL pServiceUrl, Vector pScopes, String pLanguage) {
		Hashtable scopetable = (Hashtable) iUrlScopeLanguage.get(pServiceUrl.toString());
		if (scopetable == null) { return; }

		Enumeration enumeration = pScopes.elements();
		while (enumeration.hasMoreElements()) {
			String scope = (String) enumeration.nextElement();

			Hashtable langtable = (Hashtable) scopetable.get(scope);
			if (langtable == null) {
				continue;
			}
			if (pLanguage != null) {
				langtable.remove(pLanguage);
				if (langtable.size() == 0) scopetable.remove(scope);
			} else {
				// Enumeration keys = langtable.keys();
				// while (keys.hasMoreElements()) {
				// }
				scopetable.remove(scope);
			}

			ServiceType servicetype = pServiceUrl.getServiceType();
			String key1 = scope + ':' + pLanguage + ':' + servicetype.getAbstractTypeName();
			String key2 = scope + ':' + pLanguage + ':' + servicetype.toString();

			Vector services = (Vector) iScopeLanguageType.get(key1);
			if (services != null) {
				services.remove(pServiceUrl);
				if (services.size() == 0) iScopeLanguageType.remove(key1);
			}
			if (!key1.equals(key2) && (services = (Vector) iScopeLanguageType.get(key2)) != null) {
				services.remove(pServiceUrl);
				if (services.size() == 0) iScopeLanguageType.remove(key2);
			}
		}

		if (scopetable.size() == 0) iUrlScopeLanguage.remove(pServiceUrl.toString());

	}

	public synchronized Vector findAttributes(ServiceURL pServiceUrl, Vector pScopes,
			Vector pTagList, Locale pLocale) {
		return null;
	}

	private ServiceEntry findEntry(ServiceURL pServiceUrl, Vector pScopes, String pLanguage) {
		ServiceEntry service = null;
		Hashtable scopetable = (Hashtable) iUrlScopeLanguage.get(pServiceUrl.toString());
		if (scopetable == null) return null;

		Enumeration enumeration;
		if (pScopes != null) enumeration = pScopes.elements();
		else enumeration = scopetable.keys();

		while (enumeration.hasMoreElements()) {
			String scope = (String) enumeration.nextElement();

			Hashtable langTable = (Hashtable) scopetable.get(scope);
			if (pLanguage != null) {
				service = (ServiceEntry) langTable.get(pLanguage);
				break;
			}
			Enumeration e = langTable.elements();
			service = (ServiceEntry) e.nextElement();
		}
		return service;
	}

	public synchronized Vector findServices(ServiceType pServiceType, Vector pScopes,
			String pQuery, Locale pLocale) {
		Vector result = new Vector();
		String lang = pLocale.getLanguage();
		Enumeration enumeration = pScopes.elements();
		while (enumeration.hasMoreElements()) {
			String scope = (String) enumeration.nextElement();

			Vector scopeURLs = (Vector) iScopeLanguageType.get(scope + ':' + lang + ':'
					+ pServiceType.toString());
			if (scopeURLs != null) result.addAll(scopeURLs);
		}

		// TODO filter services using query
		return result;
	}

	public synchronized Vector findServiceTypes(ServiceType pServiceType, Vector pScopes)
			throws ServiceLocationException {
		return null;
	}

	public synchronized boolean register(ServiceURL pServiceUrl, Vector pAttributeIds,
			Vector pScopes, Locale pLocale) throws ServiceLocationException {

		String lang = pLocale.getLanguage();
		// ServiceType servicetype = serviceurl.getServiceType();
		// String absTypeName = servicetype.getAbstractTypeName();

		ServiceEntry serviceentry = findEntry(pServiceUrl, null, lang);
		if (serviceentry != null) {
			deregisterEntry(pServiceUrl, pScopes, lang);
		}

		registerEntry(new ServiceEntry(pServiceUrl, pAttributeIds, pScopes, pLocale));
		return false;
	}

	private void registerEntry(ServiceEntry pEntry) {
		Hashtable scopetable = (Hashtable) iUrlScopeLanguage.get(pEntry.iServiceurl.toString());
		if (scopetable == null) {
			scopetable = new Hashtable();
			iUrlScopeLanguage.put(pEntry.iServiceurl.toString(), scopetable);
		}
		String lang = pEntry.iLocale.getLanguage();

		Enumeration enumeration = pEntry.iScopes.elements();
		while (enumeration.hasMoreElements()) {
			String scope = (String) enumeration.nextElement();

			Hashtable langtable = (Hashtable) scopetable.get(scope);
			if (langtable == null) {
				langtable = new Hashtable();
				scopetable.put(scope, langtable);
			}
			langtable.put(lang, pEntry);

			ServiceURL serviceurl = pEntry.iServiceurl;
			ServiceType servicetype = serviceurl.getServiceType();
			String key1 = scope + ':' + lang + ':' + servicetype.getAbstractTypeName();
			String key2 = scope + ':' + lang + ':' + servicetype.toString();

			Vector services = (Vector) iScopeLanguageType.get(key1);
			if (services == null) {
				services = new Vector();
				iScopeLanguageType.put(key1, services);
			}
			if (!services.contains(serviceurl)) services.add(serviceurl);

			if (!key1.equals(key2)) {
				services = (Vector) iScopeLanguageType.get(key2);

				if (services == null) {
					services = new Vector();
					iScopeLanguageType.put(key2, services);
				}
				if (!services.contains(serviceurl)) services.add(serviceurl);
			}
		}
	}

	public synchronized void updateRegistration(ServiceURL pServiceUrl, Vector pAttributeIds,
			Vector pScopes, Locale pLocale) throws ServiceLocationException {

		String lang = pLocale.getLanguage();
		ServiceEntry entry = findEntry(pServiceUrl, pScopes, lang);
		if (entry == null) { throw new ServiceLocationException(
				ServiceLocationException.INVALID_UPDATE); }
		deregisterEntry(pServiceUrl, pScopes, lang);
		ServiceEntry newEntry = new ServiceEntry(pServiceUrl, pAttributeIds, pScopes, pLocale);

		entry.merge(newEntry);
		registerEntry(entry);
	}
}
