/**
 * ServiceTable.java
 *
 * (C) Copyright IBM Corp. 2005, 2006
 *
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE COMMON PUBLIC LICENSE 
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE 
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Common Public License from
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * @author: Roberto Pineiro, IBM, roberto.pineiro@us.ibm.com  
 * @author: Chung-hao Tan, IBM ,chungtan@us.ibm.com
 * 
 * Change History
 * Flag       Date        Prog         Description
 *------------------------------------------------------------------------------- 
 * 1516246    2006-07-22  lupusalex    Integrate SLP client code
 * 1535793    2006-09-14  lupusalex    Fix&Integrate CIM&SLP configuration classes
 */

package org.sblim.slp.internal;

import java.net.InetAddress;
import java.util.Hashtable;
import java.util.Locale;
import java.util.Vector;

import org.sblim.slp.ServiceLocationException;
import org.sblim.slp.ServiceType;
import org.sblim.slp.ServiceURL;

/**
 */
public class ServiceTable {

	private static ServiceRepository cStore = new ServiceRepository();

	public ServiceTable() {
		super();
	}

	public ServiceLocationMessage deregister(SLPServiceDeregistration pServiceRegistration) {
		Vector finalAttrList = new Vector();
		ServiceLocationHeader srvlocheader = pServiceRegistration.getHeader();
		Locale locale = srvlocheader.getLocale();
		Vector scopes = srvlocheader.getScopes();
		ServiceURL serviceurl = pServiceRegistration.getUrl();

		try {
			System.out.println("Degistration:");
			System.out.println("serviceurl=" + serviceurl);
			System.out.println("attrList=" + finalAttrList);
			System.out.println("scopes=" + scopes);
			System.out.println("locale=" + locale);

			ServiceLocationMessage srvlocmsg = pServiceRegistration.makeReply();

			// TODO make sure that the new lifetime does not affect any previous
			// update rates
			cStore.deregister(serviceurl, scopes);
			return srvlocmsg;

		} catch (ServiceLocationException e) {
			return srvlocheader.makeErrorReply(e);
		} catch (RuntimeException e) {
			return srvlocheader.makeErrorReply(e);
		}
	}

	public ServiceLocationMessage findAttributes(SLPAttributeRequestMessage pAttributeMsg) {

		ServiceLocationHeader header = pAttributeMsg.getHeader();
		Vector scopes = header.getScopes();
		Locale locale = header.getLocale();

		ServiceURL serviceurl = pAttributeMsg.getUrl();
		String servicetypeStr = pAttributeMsg.getServiceType();
		Vector tagList = pAttributeMsg.getTags();
		// boolean flag = false;
		try {
			if (!supportedScopes(scopes)) throw new ServiceLocationException(
					ServiceLocationException.SCOPE_NOT_SUPPORTED);

			Vector attributeList = null;
			Hashtable authInfo = null;

			if (servicetypeStr == null) {
				if (!pAttributeMsg.getSpi().equals("") && tagList.size() > 0) throw new ServiceLocationException(
						ServiceLocationException.AUTHENTICATION_FAILED);

				attributeList = cStore.findAttributes(serviceurl, scopes, tagList, locale);

			} else {
				if (!pAttributeMsg.getSpi().equals("")) throw new ServiceLocationException(
						ServiceLocationException.AUTHENTICATION_FAILED);
				// attributeList = store.findAttributes(servicetypeStr, scopes,
				// tagList, locale);
			}

			ServiceLocationMessage srvlocmsg = pAttributeMsg.makeReply(attributeList, authInfo);

			return srvlocmsg;
		} catch (ServiceLocationException e) {
			return header.makeErrorReply(e);
		} catch (RuntimeException e) {
			return header.makeErrorReply(e);
		}
	}

	public ServiceLocationMessage findServices(SLPServiceRequestMessage pServiceMsg) {

		ServiceLocationHeader srvlocheader = pServiceMsg.getHeader();
		Locale locale = srvlocheader.getLocale();
		Vector scopes = srvlocheader.getScopes();
		ServiceType servicetype = pServiceMsg.getServiceType();
		String query = pServiceMsg.getQuery();
		// boolean flag = false;
		try {
			if (!supportedScopes(scopes)) throw new ServiceLocationException(
					ServiceLocationException.SCOPE_NOT_SUPPORTED);
			// Hashtable hashtable = _findServices(serviceType, scopes, query,
			// locale);

			Vector services = new Vector();
			ServiceURL url;

			if (servicetype.equals(SLPDefaults.SA_SERVICE_TYPE)) {
				InetAddress localhost = SLPConfig.getGlobalSLPConfig().getLocalHost();

				String hostname = localhost.getHostAddress();
				url = new ServiceURL(
						String.valueOf(SLPDefaults.SA_SERVICE_TYPE) + "://" + hostname, -1);
				;
				services.add(url);

			} else {
				services = cStore.findServices(servicetype, scopes, query, locale);
			}

			ServiceLocationMessage srvlocmsg = pServiceMsg.makeReply(services);

			return srvlocmsg;
		} catch (ServiceLocationException e) {
			return srvlocheader.makeErrorReply(e);
		} catch (RuntimeException e) {
			return srvlocheader.makeErrorReply(e);
		}
	}

	public ServiceLocationMessage findServiceTypes(SLPServiceTypeRequestMessage pTypeMsg) {
		return null;
	}

	public ServiceLocationMessage makeDAAdvert(SLPServiceRequestMessage pServiceMsg,
			InetAddress pInetAddress, SLPConfig pSlpConfig) {
		return null;
	}

	public ServiceLocationMessage makeSAAdvert(SLPServiceRequestMessage pServiceMsg,
			InetAddress pInetAddress) throws ServiceLocationException {

		ServiceLocationHeader srvlocheader = pServiceMsg.getHeader();
		int version = srvlocheader.getVersion();
		short xid = srvlocheader.getXid();
		Locale locale = srvlocheader.getLocale();
		Vector scopes = srvlocheader.getScopes();
		// String query = srvmsg.query;
		// String servicetype = srvmsg.serviceType.toString();
		// Vector availableScopes = SLPConfig.getSAOnlyScopes();

		try {
			// TODO validate security, make sure that the message can be signed

			// if (!supportedScopes(scopes) && scopes.size() > 0)
			// return null;

			if (scopes.size() <= 0) scopes = (Vector) SLPConfig.getGlobalSLPConfig().getConfiguredScopes().clone();

			String hostname = pInetAddress.getHostAddress();
			ServiceURL serviceurl = new ServiceURL(String.valueOf(SLPDefaults.SA_SERVICE_TYPE)
					+ "://" + hostname, -1);
			;

			return new SLPSAAdvert(version, xid, locale, serviceurl, SLPConfig
					.getGlobalSLPConfig().getConfiguredScopes(), new Vector());
		} catch (ServiceLocationException e) {
			return srvlocheader.makeErrorReply(e);
		} catch (RuntimeException e) {
			return srvlocheader.makeErrorReply(e);
		}
	}

	public ServiceLocationMessage register(SLPServiceRegistration pServiceRegistration) {

		ServiceLocationHeader srvlocheader = pServiceRegistration.getHeader();
		Locale locale = srvlocheader.getLocale();
		boolean isFresh = srvlocheader.isFresh();

		Vector scopes = srvlocheader.getScopes();
		ServiceURL serviceurl = pServiceRegistration.getUrl();
		// String serviceType = srvreg.serviceType;
		Vector attributeList = pServiceRegistration.getAttributes();

		short errorCode = isFresh ? ServiceLocationException.INVALID_REGISTRATION
				: ServiceLocationException.INVALID_UPDATE;
		try {
			// TODO verify urlSignature
			// TODO verify attrSignature

			if (serviceurl.getLifetime() <= 0) throw new ServiceLocationException(errorCode);

			// note that if there is urlSignature, it must be a new
			// registration, not an update
			// TODO verify scope are supported

			Vector finalAttrList = new Vector();
			// final int j = attributeList.size();
			// for (int i = 0; i < j; i++) {
			// ServiceLocationAttribute servicelocationattribute =
			// (ServiceLocationAttribute)attributeList.elementAt(i);
			// TODO remove/merge any duplicate attribute
			// }
			finalAttrList = attributeList; // TODO remove this

			boolean success = false;
			if (isFresh) {
				// TODO register
				// TODO remember to add the urlSignature and attrSignature
				success = true;
				System.out.println("New Registration:");
				success = cStore.register(serviceurl, finalAttrList, scopes, locale);
			} else {
				System.out.println("Update Registration:");
				cStore.updateRegistration(serviceurl, finalAttrList, scopes, locale);
			}
			System.out.println("serviceurl=" + serviceurl);
			System.out.println("attrList=" + finalAttrList);
			System.out.println("scopes=" + scopes);
			System.out.println("locale=" + locale);

			ServiceLocationMessage srvlocmsg = pServiceRegistration.makeReply(success);

			// TODO make sure that the new lifetime does not affect any previous
			// update rates
			return srvlocmsg;
		} catch (ServiceLocationException e) {
			return srvlocheader.makeErrorReply(e);
		} catch (RuntimeException e) {
			return srvlocheader.makeErrorReply(e);
		}
	}

	private boolean supportedScopes(Vector pScopeList) {
		Vector SAConfiguredScopes = SLPConfig.getGlobalSLPConfig().getConfiguredScopes();
		Vector SAOnlyScopes = SLPConfig.getGlobalSLPConfig().getSAOnlyScopes();

		int i = 0;
		while (i < pScopeList.size()) {
			Object scope = pScopeList.elementAt(i);

			if (!SAConfiguredScopes.contains(scope) && !SAOnlyScopes.contains(scope)) pScopeList
					.removeElementAt(i);
			else i++;
		}

		return pScopeList.size() > 0;
	}
}
