/**
 * Utils.java
 *
 * (C) Copyright IBM Corp. 2005, 2006
 *
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE COMMON PUBLIC LICENSE 
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE 
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Common Public License from
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * @author: Roberto Pineiro, IBM, roberto.pineiro@us.ibm.com  
 * @author: Chung-hao Tan, IBM ,chungtan@us.ibm.com
 * 
 * Change History
 * Flag       Date        Prog         Description
 *------------------------------------------------------------------------------- 
 * 1516246    2006-07-22  lupusalex    Integrate SLP client code
 */

package org.sblim.slp.internal;

import java.io.UnsupportedEncodingException;
import java.util.NoSuchElementException;
import java.util.StringTokenizer;
import java.util.Vector;

import org.sblim.slp.ServiceLocationException;

public abstract class Utils {

	public static void escapeScopeStrings(Vector pValueList) throws ServiceLocationException {
		int size = pValueList.size();

		for (int i = 0; i < size; i++) {
			String s = (String) pValueList.elementAt(i);
			pValueList.setElementAt(escapeAttributeString(s), i);
		}
	}

	public static void unescapeScopeStrings(Vector pValueList) throws ServiceLocationException {
		int size = pValueList.size();
		for (int i = 0; i < size; i++) {
			String s = (String) pValueList.elementAt(i);
			pValueList.setElementAt(unescapeAttributeString(s), i);
		}
	}

	public static void escapeTags(Vector pValueList) throws ServiceLocationException {
		int size = pValueList.size();
		for (int i = 0; i < size; i++) {
			Object obj = pValueList.elementAt(i);
			if (obj instanceof String) {
				String s = escapeAttributeString((String) obj);
				pValueList.setElementAt(s.trim(), i);
			} else {
				throw new IllegalArgumentException();
			}
		}
	}

	static void unescapeTags(Vector pValueList) throws ServiceLocationException {
		int size = pValueList.size();
		for (int i = 0; i < size; i++) {
			String s = (String) pValueList.elementAt(i);
			s = unescapeAttributeString(s);
			pValueList.setElementAt(s.trim(), i);
		}
	}

	public static String escapeAttributeString(String pString) throws ServiceLocationException {
		StringBuffer sb = new StringBuffer();
		int size = pString.length();
		for (int pos = 0; pos < size; pos++) {
			char c = pString.charAt(pos);

			if (isEscapable(c)) {
				sb.append('\\');
				String escaped = escapeChar(c);
				if (escaped.length() <= 1) escaped = "0" + escaped;
				sb.append(escaped);
			} else {
				sb.append(c);
			}
		}
		return sb.toString();
	}

	private static boolean isEscapable(char pChar) {
		return "(),\\!<=>~".indexOf(pChar) > -1 || (pChar >= 0 && pChar <= '\037')
				|| pChar == '\177';
	}

	public static String unescapeAttributeString(String pString) throws ServiceLocationException {
		int size = pString.length();
		StringBuffer sb = new StringBuffer(size);
		for (int pos = 0; pos < size; pos++) {
			char ch = pString.charAt(pos);
			if (ch == '\\') {
				if (pos > size - 2) throw new ServiceLocationException(
						ServiceLocationException.PARSE_ERROR);
				pos++;
				ch = unescapeChar(pString.substring(pos, pos + 2));
				pos++;
				if (!isEscapable(ch)) throw new ServiceLocationException(
						ServiceLocationException.PARSE_ERROR);
			} else if (isEscapable(ch)) throw new ServiceLocationException(
					ServiceLocationException.PARSE_ERROR);

			sb.append(ch);
		}

		return sb.toString();
	}

	public static String escapeChar(char pChar) {
		byte _chars[] = null;
		try {
			_chars = String.valueOf(pChar).getBytes("UTF8");
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		}
		int i = 0;
		if (_chars.length > 3) System.out.println("illegal UTF-8 character");

		i = _chars[0] & 0xff;
		if (_chars.length > 1) i |= (_chars[1] & 0xff) << 8;
		if (_chars.length > 2) i |= (_chars[2] & 0xff) << 16;
		String s = Integer.toHexString(i);
		return s;
	}

	public static char unescapeChar(String pString) throws ServiceLocationException {
		int value = 0;
		try {
			value = Integer.parseInt(pString, 16);
		} catch (NumberFormatException e) {
			throw new ServiceLocationException(ServiceLocationException.PARSE_ERROR, e);
		}
		String s1 = "";
		byte strBuf[] = null;

		byte b0 = (byte) (value & 0xff);
		byte b1 = (byte) (value >> 8 & 0xff);
		byte b2 = (byte) (value >> 16 & 0xff);
		byte b3 = (byte) (value >> 24 & 0xff);

		if (b3 != 0) {
			strBuf = new byte[4];
			strBuf[3] = b3;
			strBuf[2] = b2;
			strBuf[1] = b1;
			strBuf[0] = b0;
		} else if (b2 != 0) {
			strBuf = new byte[3];
			strBuf[2] = b2;
			strBuf[1] = b1;
			strBuf[0] = b0;
		} else if (b1 != 0) {
			strBuf = new byte[2];
			strBuf[1] = b1;
			strBuf[0] = b0;
		} else {
			strBuf = new byte[1];
			strBuf[0] = b0;
		}

		try {
			s1 = new String(strBuf, "UTF8");
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		}
		int length = s1.length();
		if (s1.length() > 1) throw new ServiceLocationException(
				ServiceLocationException.PARSE_ERROR);
		return length != 1 ? '\0' : s1.charAt(0);
	}

	public static Vector parseCommaSeparatedList(String pString, boolean pIgnoreParenthesis)
			throws ServiceLocationException {
		if (pString == null || pString.length() == 0) return new Vector();

		StringTokenizer stringtokenizer = new StringTokenizer(pString, ",()", true);
		try {
			int parenthesisLevel = 0;
			String subValue = "";
			Vector vector = new Vector();
			while (stringtokenizer.hasMoreElements()) {
				String token = stringtokenizer.nextToken();
				if (token.equals("(")) {
					if (!pIgnoreParenthesis) parenthesisLevel++;
					subValue = subValue + token;
				} else if (token.equals(")")) {
					if (!pIgnoreParenthesis) parenthesisLevel--;
					subValue = subValue + token;
				} else if (token.equals(",")) {
					if (parenthesisLevel != 0) {
						subValue = subValue + token;
					} else {
						if (subValue.length() <= 0) throw new ServiceLocationException(
								ServiceLocationException.PARSE_ERROR);
						vector.addElement(subValue);
						subValue = "";
					}
				} else {
					subValue = subValue + token;
				}
			}
			if (subValue.length() <= 0) throw new ServiceLocationException(
					ServiceLocationException.PARSE_ERROR);
			vector.addElement(subValue);
			if (parenthesisLevel != 0) throw new ServiceLocationException(
					ServiceLocationException.PARSE_ERROR);
			return vector;
		} catch (NoSuchElementException e) {
			e.printStackTrace();
			return null;
		}
	}

	public static byte[] unescapeByteArray(String pString) throws ServiceLocationException {
		if (!pString.startsWith("\\ff")) throw new ServiceLocationException(
				ServiceLocationException.PARSE_ERROR);
		String byteString = pString.substring(3);

		int size = byteString.length();
		int k = 0;
		int maxNumOfChars = size / 3;
		byte opaque[] = new byte[maxNumOfChars];
		for (int i = 0; i < size; i++) {
			if (byteString.charAt(i) != '\\') throw new ServiceLocationException(
					ServiceLocationException.PARSE_ERROR);
			if (i > size - 2) throw new ServiceLocationException(
					ServiceLocationException.PARSE_ERROR);
			if (k >= maxNumOfChars) throw new ServiceLocationException(
					ServiceLocationException.PARSE_ERROR);
			try {
				i++;
				opaque[k++] = (byte) (Integer.parseInt(byteString.substring(i, i + 2), 16) & 0xff);
				i++;
			} catch (NumberFormatException e) {
				throw new ServiceLocationException(ServiceLocationException.PARSE_ERROR);
			}
		}
		return opaque;
	}

	public static String vectorToCommaSeparatedList(Vector pVector) {
		int size = pVector.size();
		StringBuffer stringbuffer = new StringBuffer();
		for (int i = 0; i < size; i++) {
			String s = (String) pVector.elementAt(i);
			if (i > 0) stringbuffer.append(',');
			stringbuffer.append(s);
		}
		return stringbuffer.toString();
	}

	public static byte[] getStringBytes(String pString, String pEncoding) {
		try {
			return pString.getBytes(pEncoding);
		} catch (UnsupportedEncodingException e) {
			return new byte[0];
		}
	}
}
